@@ -27,7 +27,7 @@
 
 #include <QObject>
 #include <QtGlobal>
-#include <QtConcurrentRun>
+#include <QtConcurrent>
 #include <QThread>
 #include <QMutex>
 #include <QFuture>
@@ -44,7 +44,7 @@
 #include <QUrl>
 #include <QImage>
 #include <QChar>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QPixmapCache>
 #include <QNetworkDiskCache>
 #include <QSettings>
@@ -66,16 +66,15 @@
 #include "playlist/playlistmanager.h"
 #include "playlist/songmimedata.h"
 #include "covermanager/albumcoverloader.h"
+#include "covermanager/albumcoverloaderresult.h"
 #include "settings/collectionsettingspage.h"
 
-using std::bind;
-using std::sort;
 using std::placeholders::_1;
 using std::placeholders::_2;
 
 const char *CollectionModel::kSavedGroupingsSettingsGroup = "SavedGroupings";
 const int CollectionModel::kPrettyCoverSize = 32;
-const char *CollectionModel::kPixmapDiskCacheDir = "/pixmapcache";
+const char *CollectionModel::kPixmapDiskCacheDir = "pixmapcache";
 
 QNetworkDiskCache *CollectionModel::sIconCache = nullptr;
 
@@ -98,8 +97,6 @@ CollectionModel::CollectionModel(CollectionBackend *backend, Application *app, Q
       total_album_count_(0),
       artist_icon_(IconLoader::Load("folder-sound")),
       album_icon_(IconLoader::Load("cdcase")),
-      playlists_dir_icon_(IconLoader::Load("folder-sound")),
-      playlist_icon_(IconLoader::Load("albums")),
       init_task_id_(-1),
       use_pretty_covers_(false),
       show_dividers_(true),
@@ -108,24 +105,27 @@ CollectionModel::CollectionModel(CollectionBackend *backend, Application *app, Q
   root_->lazy_loaded = true;
 
   group_by_[0] = GroupBy_AlbumArtist;
-  group_by_[1] = GroupBy_Album;
+  group_by_[1] = GroupBy_AlbumDisc;
   group_by_[2] = GroupBy_None;
 
   cover_loader_options_.desired_height_ = kPrettyCoverSize;
   cover_loader_options_.pad_output_image_ = true;
   cover_loader_options_.scale_output_image_ = true;
 
-  if (app_)
-    connect(app_->album_cover_loader(), SIGNAL(ImageLoaded(quint64, QUrl, QImage)), SLOT(AlbumCoverLoaded(quint64, QUrl, QImage)));
+  if (app_) {
+    connect(app_->album_cover_loader(), SIGNAL(AlbumCoverLoaded(quint64, AlbumCoverLoaderResult)), SLOT(AlbumCoverLoaded(quint64, AlbumCoverLoaderResult)));
+  }
 
   QIcon nocover = IconLoader::Load("cdcase");
-  no_cover_icon_ = nocover.pixmap(nocover.availableSizes().last()).scaled(kPrettyCoverSize, kPrettyCoverSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
-  //no_cover_icon_ = QPixmap(":/pictures/noalbumart.png").scaled(kPrettyCoverSize, kPrettyCoverSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+  if (!nocover.isNull()) {
+    no_cover_icon_ = nocover.pixmap(nocover.availableSizes().last()).scaled(kPrettyCoverSize, kPrettyCoverSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+  }
 
-  // When running under gdb, all calls to this constructor came from the same thread.
-  // If this ever changes, these two lines might need to be protected by a mutex.
-  if (sIconCache == nullptr)
+  if (app_ && !sIconCache) {
     sIconCache = new QNetworkDiskCache(this);
+    sIconCache->setCacheDirectory(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + "/" + kPixmapDiskCacheDir);
+    connect(app_, SIGNAL(ClearPixmapDiskCache()), SLOT(ClearDiskCache()));
+  }
 
   connect(backend_, SIGNAL(SongsDiscovered(SongList)), SLOT(SongsDiscovered(SongList)));
   connect(backend_, SIGNAL(SongsDeleted(SongList)), SLOT(SongsDeleted(SongList)));
@@ -139,8 +139,6 @@ CollectionModel::CollectionModel(CollectionBackend *backend, Application *app, Q
   backend_->UpdateTotalArtistCountAsync();
   backend_->UpdateTotalAlbumCountAsync();
 
-  connect(app_, SIGNAL(ClearPixmapDiskCache()), SLOT(ClearDiskCache()));
-
   ReloadSettings();
 
 }
@@ -149,7 +147,7 @@ CollectionModel::~CollectionModel() {
   delete root_;
 }
 
-void CollectionModel::set_pretty_covers(bool use_pretty_covers) {
+void CollectionModel::set_pretty_covers(const bool use_pretty_covers) {
 
   if (use_pretty_covers != use_pretty_covers_) {
     use_pretty_covers_ = use_pretty_covers;
@@ -157,7 +155,7 @@ void CollectionModel::set_pretty_covers(bool use_pretty_covers) {
   }
 }
 
-void CollectionModel::set_show_dividers(bool show_dividers) {
+void CollectionModel::set_show_dividers(const bool show_dividers) {
 
   if (show_dividers != show_dividers_) {
     show_dividers_ = show_dividers;
@@ -187,19 +185,21 @@ void CollectionModel::ReloadSettings() {
 
   use_disk_cache_ = s.value(CollectionSettingsPage::kSettingsDiskCacheEnable, false).toBool();
 
-  if (!use_disk_cache_) {
-    sIconCache->clear();
+  QPixmapCache::setCacheLimit(MaximumCacheSize(&s, CollectionSettingsPage::kSettingsCacheSize, CollectionSettingsPage::kSettingsCacheSizeUnit, CollectionSettingsPage::kSettingsCacheSizeDefault) / 1024);
+
+  if (sIconCache) {
+    sIconCache->setMaximumCacheSize(MaximumCacheSize(&s, CollectionSettingsPage::kSettingsDiskCacheSize, CollectionSettingsPage::kSettingsDiskCacheSizeUnit, CollectionSettingsPage::kSettingsDiskCacheSizeDefault));
   }
 
-  sIconCache->setCacheDirectory(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + kPixmapDiskCacheDir);
-  sIconCache->setMaximumCacheSize(MaximumCacheSize(&s, CollectionSettingsPage::kSettingsDiskCacheSize, CollectionSettingsPage::kSettingsDiskCacheSizeUnit));
-
-  QPixmapCache::setCacheLimit(MaximumCacheSize(&s, CollectionSettingsPage::kSettingsCacheSize, CollectionSettingsPage::kSettingsCacheSizeUnit) / 1024);
-
   s.endGroup();
+
+  if (!use_disk_cache_) {
+    ClearDiskCache();
+  }
+
 }
 
-void CollectionModel::Init(bool async) {
+void CollectionModel::Init(const bool async) {
 
   if (async) {
     // Show a loading indicator in the model.
@@ -338,7 +338,7 @@ void CollectionModel::SongsSlightlyChanged(const SongList &songs) {
 
 }
 
-CollectionItem *CollectionModel::CreateCompilationArtistNode(bool signal, CollectionItem *parent) {
+CollectionItem *CollectionModel::CreateCompilationArtistNode(const bool signal, CollectionItem *parent) {
 
   if (signal) beginInsertRows(ItemToIndex(parent), parent->children.count(), parent->children.count());
 
@@ -354,7 +354,7 @@ CollectionItem *CollectionModel::CreateCompilationArtistNode(bool signal, Collec
 
 }
 
-QString CollectionModel::DividerKey(GroupBy type, CollectionItem *item) const {
+QString CollectionModel::DividerKey(const GroupBy type, CollectionItem *item) const {
 
   // Items which are to be grouped under the same divider must produce the same divider key.  This will only get called for top-level items.
 
@@ -408,7 +408,7 @@ QString CollectionModel::DividerKey(GroupBy type, CollectionItem *item) const {
 
 }
 
-QString CollectionModel::DividerDisplayText(GroupBy type, const QString &key) const {
+QString CollectionModel::DividerDisplayText(const GroupBy type, const QString &key) const {
 
   // Pretty display text for the dividers.
 
@@ -510,7 +510,7 @@ void CollectionModel::SongsDeleted(const SongList &songs) {
       // Remove from pixmap cache
       const QString cache_key = AlbumIconPixmapCacheKey(ItemToIndex(node));
       QPixmapCache::remove(cache_key);
-      if (use_disk_cache_) sIconCache->remove(QUrl(cache_key));
+      if (use_disk_cache_ && sIconCache) sIconCache->remove(QUrl(cache_key));
       if (pending_cache_keys_.contains(cache_key)) {
         pending_cache_keys_.remove(cache_key);
       }
@@ -585,13 +585,13 @@ QVariant CollectionModel::AlbumIcon(const QModelIndex &idx) {
   }
 
   // Try to load it from the disk cache
-  if (use_disk_cache_) {
+  if (use_disk_cache_ && sIconCache) {
     std::unique_ptr<QIODevice> cache(sIconCache->data(QUrl(cache_key)));
     if (cache) {
-      QImage cached_pixmap;
-      if (cached_pixmap.load(cache.get(), "XPM")) {
-        QPixmapCache::insert(cache_key, QPixmap::fromImage(cached_pixmap));
-        return QPixmap::fromImage(cached_pixmap);
+      QImage cached_image;
+      if (cached_image.load(cache.get(), "XPM")) {
+        QPixmapCache::insert(cache_key, QPixmap::fromImage(cached_image));
+        return QPixmap::fromImage(cached_image);
       }
     }
   }
@@ -613,9 +613,7 @@ QVariant CollectionModel::AlbumIcon(const QModelIndex &idx) {
 
 }
 
-void CollectionModel::AlbumCoverLoaded(const quint64 id, const QUrl &cover_url, const QImage &image) {
-
-  Q_UNUSED(cover_url);
+void CollectionModel::AlbumCoverLoaded(const quint64 id, const AlbumCoverLoaderResult &result) {
 
   if (!pending_art_.contains(id)) return;
 
@@ -628,26 +626,26 @@ void CollectionModel::AlbumCoverLoaded(const quint64 id, const QUrl &cover_url,
   pending_cache_keys_.remove(cache_key);
 
   // Insert this image in the cache.
-  if (image.isNull()) {
+  if (result.image_scaled.isNull()) {
     // Set the no_cover image so we don't continually try to load art.
     QPixmapCache::insert(cache_key, no_cover_icon_);
   }
   else {
     QPixmap image_pixmap;
-    image_pixmap = QPixmap::fromImage(image);
+    image_pixmap = QPixmap::fromImage(result.image_scaled);
     QPixmapCache::insert(cache_key, image_pixmap);
   }
 
   // If we have a valid cover not already in the disk cache
-  if (use_disk_cache_) {
+  if (use_disk_cache_ && sIconCache) {
     std::unique_ptr<QIODevice> cached_img(sIconCache->data(QUrl(cache_key)));
-    if (!cached_img && !image.isNull()) {
+    if (!cached_img && !result.image_scaled.isNull()) {
       QNetworkCacheMetaData item_metadata;
       item_metadata.setSaveToDisk(true);
       item_metadata.setUrl(QUrl(cache_key));
-      QIODevice* cache = sIconCache->prepare(item_metadata);
+      QIODevice *cache = sIconCache->prepare(item_metadata);
       if (cache) {
-        image.save(cache, "XPM");
+        result.image_scaled.save(cache, "XPM");
         sIconCache->insert(cache);
       }
     }
@@ -660,7 +658,7 @@ void CollectionModel::AlbumCoverLoaded(const quint64 id, const QUrl &cover_url,
 
 }
 
-QVariant CollectionModel::data(const QModelIndex &idx, int role) const {
+QVariant CollectionModel::data(const QModelIndex &idx, const int role) const {
 
   const CollectionItem *item = IndexToItem(idx);
 
@@ -672,11 +670,7 @@ QVariant CollectionModel::data(const QModelIndex &idx, int role) const {
     bool is_album_node = false;
     if (role == Qt::DecorationRole && item->type == CollectionItem::Type_Container) {
       GroupBy container_type = group_by_[item->container_level];
-      is_album_node = container_type == GroupBy_Album ||
-                      container_type == GroupBy_AlbumDisc ||
-                      container_type == GroupBy_YearAlbum ||
-                      container_type == GroupBy_YearAlbumDisc ||
-                      container_type == GroupBy_OriginalYearAlbum;
+      is_album_node = IsAlbumGrouping(container_type);
     }
     if (is_album_node) {
       // It has const behaviour some of the time - that's ok right?
@@ -688,7 +682,7 @@ QVariant CollectionModel::data(const QModelIndex &idx, int role) const {
 
 }
 
-QVariant CollectionModel::data(const CollectionItem *item, int role) const {
+QVariant CollectionModel::data(const CollectionItem *item, const int role) const {
 
   GroupBy container_type = item->type == CollectionItem::Type_Container ? group_by_[item->container_level] : GroupBy_None;
 
@@ -699,8 +693,6 @@ QVariant CollectionModel::data(const CollectionItem *item, int role) const {
 
     case Qt::DecorationRole:
       switch (item->type) {
-        case CollectionItem::Type_PlaylistContainer:
-          return playlists_dir_icon_;
         case CollectionItem::Type_Container:
           switch (container_type) {
             case GroupBy_Album:
@@ -828,7 +820,7 @@ CollectionModel::QueryResult CollectionModel::RunQuery(CollectionItem *parent) {
 
 }
 
-void CollectionModel::PostQuery(CollectionItem *parent, const CollectionModel::QueryResult &result, bool signal) {
+void CollectionModel::PostQuery(CollectionItem *parent, const CollectionModel::QueryResult &result, const bool signal) {
 
   // Information about what we want the children to be
   int child_level = parent == root_ ? 0 : parent->container_level + 1;
@@ -852,7 +844,7 @@ void CollectionModel::PostQuery(CollectionItem *parent, const CollectionModel::Q
 
 }
 
-void CollectionModel::LazyPopulate(CollectionItem *parent, bool signal) {
+void CollectionModel::LazyPopulate(CollectionItem *parent, const bool signal) {
 
   if (parent->lazy_loaded) return;
   parent->lazy_loaded = true;
@@ -863,9 +855,8 @@ void CollectionModel::LazyPopulate(CollectionItem *parent, bool signal) {
 }
 
 void CollectionModel::ResetAsync() {
-  QFuture<CollectionModel::QueryResult> future = QtConcurrent::run(this, &CollectionModel::RunQuery, root_);
+  QFuture<CollectionModel::QueryResult> future = QtConcurrent::run(std::bind(&CollectionModel::RunQuery, this, root_));
   NewClosure(future, this, SLOT(ResetAsyncQueryFinished(QFuture<CollectionModel::QueryResult>)), future);
-
 }
 
 void CollectionModel::ResetAsyncQueryFinished(QFuture<CollectionModel::QueryResult> future) {
@@ -920,7 +911,7 @@ void CollectionModel::Reset() {
 
 }
 
-void CollectionModel::InitQuery(GroupBy type, CollectionQuery *q) {
+void CollectionModel::InitQuery(const GroupBy type, CollectionQuery *q) {
 
   // Say what type of thing we want to get back from the database.
   switch (type) {
@@ -988,7 +979,7 @@ void CollectionModel::InitQuery(GroupBy type, CollectionQuery *q) {
 
 }
 
-void CollectionModel::FilterQuery(GroupBy type, CollectionItem *item, CollectionQuery *q) {
+void CollectionModel::FilterQuery(const GroupBy type, CollectionItem *item, CollectionQuery *q) {
 
   // Say how we want the query to be filtered.  This is done once for each parent going up the tree.
 
@@ -1081,7 +1072,7 @@ void CollectionModel::FilterQuery(GroupBy type, CollectionItem *item, Collection
 
 }
 
-CollectionItem *CollectionModel::InitItem(GroupBy type, bool signal, CollectionItem *parent, int container_level) {
+CollectionItem *CollectionModel::InitItem(const GroupBy type, const bool signal, CollectionItem *parent, const int container_level) {
 
   CollectionItem::Type item_type = type == GroupBy_None ? CollectionItem::Type_Song : CollectionItem::Type_Container;
 
@@ -1096,7 +1087,7 @@ CollectionItem *CollectionModel::InitItem(GroupBy type, bool signal, CollectionI
 
 }
 
-CollectionItem *CollectionModel::ItemFromQuery(GroupBy type, bool signal, bool create_divider, CollectionItem *parent, const SqlRow &row, int container_level) {
+CollectionItem *CollectionModel::ItemFromQuery(const GroupBy type, const bool signal, const bool create_divider, CollectionItem *parent, const SqlRow &row, const int container_level) {
 
   CollectionItem *item = InitItem(type, signal, parent, container_level);
 
@@ -1219,7 +1210,12 @@ CollectionItem *CollectionModel::ItemFromQuery(GroupBy type, bool signal, bool c
       item->metadata.InitFromQuery(row, true);
       item->key = item->metadata.title();
       item->display_text = item->metadata.TitleWithCompilationArtist();
-      item->sort_text = SortTextForSong(item->metadata);
+      if (item->container_level == 1 && !IsAlbumGrouping(group_by_[0])) {
+        item->sort_text = SortText(item->metadata.title());
+      }
+      else {
+        item->sort_text = SortTextForSong(item->metadata);
+      }
       break;
   }
 
@@ -1229,7 +1225,7 @@ CollectionItem *CollectionModel::ItemFromQuery(GroupBy type, bool signal, bool c
 
 }
 
-CollectionItem *CollectionModel::ItemFromSong(GroupBy type, bool signal, bool create_divider, CollectionItem *parent, const Song &s, int container_level) {
+CollectionItem *CollectionModel::ItemFromSong(const GroupBy type, const bool signal, const bool create_divider, CollectionItem *parent, const Song &s, const int container_level) {
 
   CollectionItem *item = InitItem(type, signal, parent, container_level);
 
@@ -1360,7 +1356,12 @@ CollectionItem *CollectionModel::ItemFromSong(GroupBy type, bool signal, bool cr
       item->metadata = s;
       item->key = s.title();
       item->display_text = s.TitleWithCompilationArtist();
-      item->sort_text = SortTextForSong(s);
+      if (item->container_level == 1 && !IsAlbumGrouping(group_by_[0])) {
+        item->sort_text = SortText(s.title());
+      }
+      else {
+        item->sort_text = SortTextForSong(s);
+      }
       break;
   }
 
@@ -1371,7 +1372,7 @@ CollectionItem *CollectionModel::ItemFromSong(GroupBy type, bool signal, bool cr
 
 }
 
-void CollectionModel::FinishItem(GroupBy type, bool signal, bool create_divider, CollectionItem *parent, CollectionItem *item) {
+void CollectionModel::FinishItem(const GroupBy type, const bool signal, const bool create_divider, CollectionItem *parent, CollectionItem *item) {
 
   if (type == GroupBy_None) item->lazy_loaded = true;
 
@@ -1415,7 +1416,7 @@ QString CollectionModel::PrettyYearAlbum(const int year, const QString &album) {
 
 QString CollectionModel::PrettyAlbumDisc(const QString &album, const int disc) {
 
-  if (disc <= 0 || album.contains(QRegExp(Song::kAlbumRemoveDisc))) return TextOrUnknown(album);
+  if (disc <= 0 || album.contains(QRegularExpression(Song::kAlbumRemoveDisc))) return TextOrUnknown(album);
   else return TextOrUnknown(album) + " - (Disc " + QString::number(disc) + ")";
 
 }
@@ -1427,7 +1428,7 @@ QString CollectionModel::PrettyYearAlbumDisc(const int year, const QString &albu
   if (year <= 0) str = TextOrUnknown(album);
   else str = QString::number(year) + " - " + TextOrUnknown(album);
 
-  if (!album.contains(QRegExp(Song::kAlbumRemoveDisc)) && disc > 0) str += " - (Disc " + QString::number(disc) + ")";
+  if (!album.contains(QRegularExpression(Song::kAlbumRemoveDisc)) && disc > 0) str += " - (Disc " + QString::number(disc) + ")";
 
   return str;
 
@@ -1441,7 +1442,7 @@ QString CollectionModel::SortText(QString text) {
   else {
     text = text.toLower();
   }
-  text = text.remove(QRegExp("[^\\w ]"));
+  text = text.remove(QRegularExpression("[^\\w ]", QRegularExpression::UseUnicodePropertiesOption));
 
   return text;
 
@@ -1463,19 +1464,19 @@ QString CollectionModel::SortTextForArtist(QString artist) {
 
 }
 
-QString CollectionModel::SortTextForNumber(int number) {
+QString CollectionModel::SortTextForNumber(const int number) {
 
   return QString("%1").arg(number, 4, 10, QChar('0'));
 }
 
-QString CollectionModel::SortTextForYear(int year) {
+QString CollectionModel::SortTextForYear(const int year) {
 
   QString str = QString::number(year);
   return QString("0").repeated(qMax(0, 4 - str.length())) + str;
 
 }
 
-QString CollectionModel::SortTextForBitrate(int bitrate) {
+QString CollectionModel::SortTextForBitrate(const int bitrate) {
 
   QString str = QString::number(bitrate);
   return QString("0").repeated(qMax(0, 3 - str.length())) + str;
@@ -1541,9 +1542,9 @@ bool CollectionModel::CompareItems(const CollectionItem *a, const CollectionItem
 
 }
 
-int CollectionModel::MaximumCacheSize(QSettings *s, const char *size_id, const char *size_unit_id) const {
+int CollectionModel::MaximumCacheSize(QSettings *s, const char *size_id, const char *size_unit_id, const int cache_size_default) const {
 
-  int size = s->value(size_id, 80).toInt();
+  int size = s->value(size_id, cache_size_default).toInt();
   int unit = s->value(size_unit_id, CollectionSettingsPage::CacheSizeUnit::CacheSizeUnit_MB).toInt() + 1;
 
   do {
@@ -1552,6 +1553,7 @@ int CollectionModel::MaximumCacheSize(QSettings *s, const char *size_id, const c
   } while (unit > 0);
 
   return size;
+
 }
 
 void CollectionModel::GetChildSongs(CollectionItem *item, QList<QUrl> *urls, SongList *songs, QSet<int> *song_ids) const {
@@ -1599,7 +1601,7 @@ SongList CollectionModel::GetChildSongs(const QModelIndex &idx) const {
   return GetChildSongs(QModelIndexList() << idx);
 }
 
-void CollectionModel::SetFilterAge(int age) {
+void CollectionModel::SetFilterAge(const int age) {
   query_options_.set_max_age(age);
   ResetAsync();
 }
@@ -1634,7 +1636,7 @@ void CollectionModel::SetGroupBy(const Grouping &g) {
 
 }
 
-const CollectionModel::GroupBy &CollectionModel::Grouping::operator[](int i) const {
+const CollectionModel::GroupBy &CollectionModel::Grouping::operator[](const int i) const {
 
   switch (i) {
     case 0: return first;
@@ -1646,7 +1648,7 @@ const CollectionModel::GroupBy &CollectionModel::Grouping::operator[](int i) con
 
 }
 
-CollectionModel::GroupBy &CollectionModel::Grouping::operator[](int i) {
+CollectionModel::GroupBy &CollectionModel::Grouping::operator[](const int i) {
 
   switch (i) {
     case 0: return first;
@@ -1660,21 +1662,21 @@ CollectionModel::GroupBy &CollectionModel::Grouping::operator[](int i) {
 }
 
 
-void CollectionModel::TotalSongCountUpdatedSlot(int count) {
+void CollectionModel::TotalSongCountUpdatedSlot(const int count) {
 
   total_song_count_ = count;
   emit TotalSongCountUpdated(count);
 
 }
 
-void CollectionModel::TotalArtistCountUpdatedSlot(int count) {
+void CollectionModel::TotalArtistCountUpdatedSlot(const int count) {
 
   total_artist_count_ = count;
   emit TotalArtistCountUpdated(count);
 
 }
 
-void CollectionModel::TotalAlbumCountUpdatedSlot(int count) {
+void CollectionModel::TotalAlbumCountUpdatedSlot(const int count) {
 
   total_album_count_ = count;
   emit TotalAlbumCountUpdated(count);
@@ -1682,7 +1684,7 @@ void CollectionModel::TotalAlbumCountUpdatedSlot(int count) {
 }
 
 void CollectionModel::ClearDiskCache() {
-  sIconCache->clear();
+  if (sIconCache) sIconCache->clear();
 }
 
 QDataStream &operator<<(QDataStream &s, const CollectionModel::Grouping &g) {
