@@ -30,18 +30,20 @@
 #include <QObject>
 #include <QFile>
 #include <QFileInfo>
+#include <QDir>
 #include <QSharedData>
 #include <QHash>
 #include <QByteArray>
 #include <QVariant>
 #include <QString>
 #include <QStringList>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QUrl>
 #include <QImage>
 #include <QIcon>
 #include <QTextCodec>
 #include <QSqlQuery>
+#include <QStandardPaths>
 #include <QtDebug>
 
 #ifdef HAVE_LIBGPOD
@@ -66,7 +68,6 @@
 #include "covermanager/albumcoverloader.h"
 #include "tagreadermessages.pb.h"
 
-using std::sort;
 #ifndef USE_SYSTEM_TAGLIB
 using namespace Strawberry_TagLib;
 #endif
@@ -147,9 +148,9 @@ const QString Song::kFtsUpdateSpec = Utilities::Updateify(Song::kFtsColumns).joi
 const QString Song::kManuallyUnsetCover = "(unset)";
 const QString Song::kEmbeddedCover = "(embedded)";
 
-const QRegExp Song::kAlbumRemoveDisc(" ?-? ((\\(|\\[)?)(Disc|CD) ?([0-9]{1,2})((\\)|\\])?)$");
-const QRegExp Song::kAlbumRemoveMisc(" ?-? ((\\(|\\[)?)(Remastered|([0-9]{1,4}) *Remaster) ?((\\)|\\])?)$");
-const QRegExp Song::kTitleRemoveMisc(" ?-? ((\\(|\\[)?)(Remastered|Live|Remastered Version|([0-9]{1,4}) *Remaster) ?((\\)|\\])?)$");
+const QRegularExpression Song::kAlbumRemoveDisc(" ?-? ((\\(|\\[)?)(Disc|CD) ?([0-9]{1,2})((\\)|\\])?)$");
+const QRegularExpression Song::kAlbumRemoveMisc(" ?-? ((\\(|\\[)?)(Remastered|([0-9]{1,4}) *Remaster) ?((\\)|\\])?)$");
+const QRegularExpression Song::kTitleRemoveMisc(" ?-? ((\\(|\\[)?)(Remastered|Live|Remastered Version|([0-9]{1,4}) *Remaster) ?((\\)|\\])?)$");
 const QString Song::kVariousArtists("various artists");
 
 const QStringList Song::kArticles = QStringList() << "the " << "a " << "an ";
@@ -181,9 +182,9 @@ struct Song::Private : public QSharedData {
   QString comment_;
   QString lyrics_;
 
-  qint64 artist_id_;
+  QString artist_id_;
   QString album_id_;
-  qint64 song_id_;
+  QString song_id_;
 
   qint64 beginning_;
   qint64 end_;
@@ -198,8 +199,8 @@ struct Song::Private : public QSharedData {
   QUrl url_;
   FileType filetype_;
   int filesize_;
-  int mtime_;
-  int ctime_;
+  qint64 mtime_;
+  qint64 ctime_;
   bool unavailable_;
 
   int playcount_;
@@ -235,9 +236,6 @@ Song::Private::Private(Song::Source source)
       originalyear_(-1),
       compilation_(false),
 
-      artist_id_(-1),
-      song_id_(-1),
-
       beginning_(0),
       end_(-1),
 
@@ -279,9 +277,9 @@ bool Song::is_valid() const { return d->valid_; }
 bool Song::is_unavailable() const { return d->unavailable_; }
 int Song::id() const { return d->id_; }
 
-qint64 Song::artist_id() const { return d->artist_id_; }
+QString Song::artist_id() const { return d->artist_id_.isNull() ? "" : d->artist_id_; }
 QString Song::album_id() const { return d->album_id_.isNull() ? "" : d->album_id_; }
-qint64 Song::song_id() const { return d->song_id_; }
+QString Song::song_id() const { return d->song_id_.isNull() ? "" : d->song_id_; }
 
 const QString &Song::title() const { return d->title_; }
 const QString &Song::title_sortable() const { return d->title_sortable_; }
@@ -324,8 +322,8 @@ const QUrl &Song::url() const { return d->url_; }
 const QString &Song::basefilename() const { return d->basefilename_; }
 Song::FileType Song::filetype() const { return d->filetype_; }
 int Song::filesize() const { return d->filesize_; }
-uint Song::mtime() const { return d->mtime_; }
-uint Song::ctime() const { return d->ctime_; }
+qint64 Song::mtime() const { return d->mtime_; }
+qint64 Song::ctime() const { return d->ctime_; }
 
 int Song::playcount() const { return d->playcount_; }
 int Song::skipcount() const { return d->skipcount_; }
@@ -350,13 +348,14 @@ const QString &Song::cue_path() const { return d->cue_path_; }
 bool Song::has_cue() const { return !d->cue_path_.isEmpty(); }
 
 bool Song::is_collection_song() const { return d->source_ == Source_Collection; }
-bool Song::is_metadata_good() const { return !d->title_.isEmpty() && !d->artist_.isEmpty() && !d->url_.isEmpty() && d->end_ > 0; }
+bool Song::is_metadata_good() const { return !d->url_.isEmpty() && !d->artist_.isEmpty() && !d->title_.isEmpty(); }
 bool Song::is_stream() const { return d->source_ == Source_Stream || d->source_ == Source_Tidal || d->source_ == Source_Subsonic || d->source_ == Source_Qobuz; }
 bool Song::is_cdda() const { return d->source_ == Source_CDDA; }
 bool Song::is_compilation() const { return (d->compilation_ || d->compilation_detected_ || d->compilation_on_) && !d->compilation_off_; }
 
 bool Song::art_automatic_is_valid() const {
-  return (
+  return !d->art_automatic_.isEmpty() &&
+         (
          (d->art_automatic_.path() == kManuallyUnsetCover) ||
          (d->art_automatic_.path() == kEmbeddedCover) ||
          (d->art_automatic_.isValid() && !d->art_automatic_.isLocalFile()) ||
@@ -366,7 +365,8 @@ bool Song::art_automatic_is_valid() const {
 }
 
 bool Song::art_manual_is_valid() const {
-  return (
+  return !d->art_manual_.isEmpty() &&
+         (
          (d->art_manual_.path() == kManuallyUnsetCover) ||
          (d->art_manual_.path() == kEmbeddedCover) ||
          (d->art_manual_.isValid() && !d->art_manual_.isLocalFile()) ||
@@ -380,10 +380,9 @@ const QString &Song::error() const { return d->error_; }
 void Song::set_id(int id) { d->id_ = id; }
 void Song::set_valid(bool v) { d->valid_ = v; }
 
-void Song::set_artist_id(qint64 v) { d->artist_id_ = v; }
-void Song::set_album_id(qint64 v) { d->album_id_ = QString::number(v); }
+void Song::set_artist_id(const QString &v) { d->artist_id_ = v; }
 void Song::set_album_id(const QString &v) { d->album_id_ = v; }
-void Song::set_song_id(qint64 v) { d->song_id_ = v; }
+void Song::set_song_id(const QString &v) { d->song_id_ = v; }
 
 QString Song::sortable(const QString &v) const {
 
@@ -429,8 +428,8 @@ void Song::set_url(const QUrl &v) { d->url_ = v; }
 void Song::set_basefilename(const QString &v) { d->basefilename_ = v; }
 void Song::set_filetype(FileType v) { d->filetype_ = v; }
 void Song::set_filesize(int v) { d->filesize_ = v; }
-void Song::set_mtime(int v) { d->mtime_ = v; }
-void Song::set_ctime(int v) { d->ctime_ = v; }
+void Song::set_mtime(qint64 v) { d->mtime_ = v; }
+void Song::set_ctime(qint64 v) { d->ctime_ = v; }
 void Song::set_unavailable(bool v) { d->unavailable_ = v; }
 
 void Song::set_playcount(int v) { d->playcount_ = v; }
@@ -659,7 +658,30 @@ Song::FileType Song::FiletypeByExtension(const QString &ext) {
 
 }
 
-int CompareSongsName(const Song &song1, const Song &song2) {
+QString Song::ImageCacheDir(const Song::Source source) {
+
+  switch (source) {
+    case Song::Source_Collection:
+      return QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/collectionalbumcovers";
+    case Song::Source_Subsonic:
+      return QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/subsonicalbumcovers";
+    case Song::Source_Tidal:
+      return QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/tidalalbumcovers";
+    case Song::Source_Qobuz:
+      return QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/qobuzalbumcovers";
+    case Song::Source_LocalFile:
+    case Song::Source_CDDA:
+    case Song::Source_Device:
+    case Song::Source_Stream:
+    case Song::Source_Unknown:
+      return QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + "/albumcovers";
+  }
+
+  return QString();
+
+}
+
+int Song::CompareSongsName(const Song &song1, const Song &song2) {
   return song1.PrettyTitleWithArtist().localeAwareCompare(song2.PrettyTitleWithArtist()) < 0;
 }
 
@@ -860,13 +882,13 @@ void Song::InitFromQuery(const SqlRow &q, bool reliable_metadata, int col) {
     }
 
     else if (Song::kColumns.value(i) == "artist_id") {
-      d->artist_id_ = tolonglong(x);
+      d->artist_id_ = tostr(x);
     }
     else if (Song::kColumns.value(i) == "album_id") {
       d->album_id_ = tostr(x);
     }
     else if (Song::kColumns.value(i) == "song_id") {
-      d->song_id_ = tolonglong(x);
+      d->song_id_ = tostr(x);
     }
 
     else if (Song::kColumns.value(i) == "beginning") {
@@ -903,10 +925,10 @@ void Song::InitFromQuery(const SqlRow &q, bool reliable_metadata, int col) {
       d->filesize_ = toint(x);
     }
     else if (Song::kColumns.value(i) == "mtime") {
-      d->mtime_ = toint(x);
+      d->mtime_ = tolonglong(x);
     }
     else if (Song::kColumns.value(i) == "ctime") {
-      d->ctime_ = toint(x);
+      d->ctime_ = tolonglong(x);
     }
     else if (Song::kColumns.value(i) == "unavailable") {
       d->unavailable_ = q.value(x).toBool();
@@ -936,7 +958,7 @@ void Song::InitFromQuery(const SqlRow &q, bool reliable_metadata, int col) {
 
     else if (Song::kColumns.value(i) == "art_automatic") {
       QString art_automatic = tostr(x);
-      if (art_automatic.contains(QRegExp("..+:.*"))) {
+      if (art_automatic.contains(QRegularExpression("..+:.*"))) {
         set_art_automatic(QUrl::fromEncoded(art_automatic.toUtf8()));
       }
       else {
@@ -945,7 +967,7 @@ void Song::InitFromQuery(const SqlRow &q, bool reliable_metadata, int col) {
     }
     else if (Song::kColumns.value(i) == "art_manual") {
       QString art_manual = tostr(x);
-      if (art_manual.contains(QRegExp("..+:.*"))) {
+      if (art_manual.contains(QRegularExpression("..+:.*"))) {
         set_art_manual(QUrl::fromEncoded(art_manual.toUtf8()));
       }
       else {
@@ -1001,16 +1023,24 @@ void Song::InitFromFilePartial(const QString &filename) {
 
 void Song::InitArtManual() {
 
-  QString album = d->album_;
+  QString album = effective_album();
   album.remove(Song::kAlbumRemoveDisc);
 
   // If we don't have an art, check if we have one in the cache
-  if (d->art_manual_.isEmpty() && d->art_automatic_.isEmpty()) {
+  if (d->art_manual_.isEmpty() && d->art_automatic_.isEmpty() && !effective_albumartist().isEmpty() && !album.isEmpty()) {
     QString filename(Utilities::Sha1CoverHash(effective_albumartist(), album).toHex() + ".jpg");
-    QString path(AlbumCoverLoader::ImageCacheDir(d->source_) + "/" + filename);
+    QString path(ImageCacheDir(d->source_) + "/" + filename);
     if (QFile::exists(path)) {
       d->art_manual_ = QUrl::fromLocalFile(path);
     }
+    else if (d->url_.isLocalFile()) { // Pick the first image file in the album directory.
+      QFileInfo file(d->url_.toLocalFile());
+      QDir dir(file.path());
+      QStringList files = dir.entryList(QStringList() << "*.jpg" << "*.png" << "*.gif" << "*.jpeg", QDir::Files|QDir::Readable, QDir::Name);
+      if (files.count() > 0) {
+        d->art_manual_ = QUrl::fromLocalFile(file.path() + QDir::separator() + files.first());
+      }
+    }
   }
 
 }
@@ -1243,9 +1273,9 @@ bool Song::MergeFromSimpleMetaBundle(const Engine::SimpleMetaBundle &bundle) {
 
 void Song::BindToQuery(QSqlQuery *query) const {
 
-#define strval(x) (x.isNull() ? "" : x)
-#define intval(x) (x <= 0 ? -1 : x)
-#define notnullintval(x) (x == -1 ? QVariant() : x)
+#define strval(x) ((x).isNull() ? "" : (x))
+#define intval(x) ((x) <= 0 ? -1 : (x))
+#define notnullintval(x) ((x) == -1 ? QVariant() : (x))
 
   // Remember to bind these in the same order as kBindSpec
 
@@ -1265,9 +1295,9 @@ void Song::BindToQuery(QSqlQuery *query) const {
   query->bindValue(":comment", strval(d->comment_));
   query->bindValue(":lyrics", strval(d->lyrics_));
 
-  query->bindValue(":artist_id", intval(d->artist_id_));
+  query->bindValue(":artist_id", strval(d->artist_id_));
   query->bindValue(":album_id", strval(d->album_id_));
-  query->bindValue(":song_id", intval(d->song_id_));
+  query->bindValue(":song_id", strval(d->song_id_));
 
   query->bindValue(":beginning", d->beginning_);
   query->bindValue(":length", intval(length_nanosec()));
