@@ -22,7 +22,7 @@
 #include "config.h"
 
 #include <memory>
-#include <stdlib.h>
+#include <cstdlib>
 #include <iconv.h>
 
 #include <QtGlobal>
@@ -50,12 +50,14 @@
 #include <QString>
 #include <QStringList>
 #include <QUrl>
-#include <QRegExp>
+#include <QRegularExpression>
+#include <QRegularExpressionMatch>
 #include <QTcpServer>
 #include <QTemporaryFile>
 #include <QPoint>
 #include <QRect>
 #include <QSize>
+#include <QColor>
 #include <QMetaEnum>
 #include <QXmlStreamReader>
 #include <QSettings>
@@ -63,8 +65,11 @@
 #include <QMessageBox>
 #include <QNetworkInterface>
 #include <QtDebug>
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+#  include <QRandomGenerator>
+#endif
 
-#include <stdio.h>
+#include <cstdio>
 
 #ifdef Q_OS_LINUX
 #  include <unistd.h>
@@ -153,7 +158,7 @@ QString WordyTimeNanosec(qint64 nanoseconds) {
 QString Ago(int seconds_since_epoch, const QLocale &locale) {
 
   const QDateTime now = QDateTime::currentDateTime();
-  const QDateTime then = QDateTime::fromTime_t(seconds_since_epoch);
+  const QDateTime then = QDateTime::fromSecsSinceEpoch(seconds_since_epoch);
   const int days_ago = then.date().daysTo(now.date());
   const QString time = then.time().toString(locale.timeFormat(QLocale::ShortFormat));
 
@@ -332,7 +337,10 @@ QString ColorToRgba(const QColor &c) {
 }
 
 #if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
-void OpenInFileManager(const QString &path) {
+void OpenInFileManager(const QString path, const QUrl &url);
+void OpenInFileManager(const QString path, const QUrl &url) {
+
+  if (!url.isLocalFile()) return;
 
   QProcess proc;
   proc.start("xdg-mime", QStringList() << "query" << "default" << "inode/directory");
@@ -365,27 +373,27 @@ void OpenInFileManager(const QString &path) {
     command_params.removeAt(command_params.indexOf("%U"));
   }
 
+  if (command.startsWith("/usr/bin/")) {
+    command = command.split("/").last();
+  }
+
   if (command.isEmpty() || command == "exo-open") {
-    QFileInfo info(path);
-    if (!info.exists()) return;
-    QString directory = info.dir().path();
-    if (directory.isEmpty()) return;
-    QDesktopServices::openUrl(QUrl::fromLocalFile(directory));
+    QDesktopServices::openUrl(QUrl::fromLocalFile(path));
   }
   else if (command.startsWith("nautilus")) {
-    proc.startDetached(command, QStringList() << command_params << "--select" << path);
+    proc.startDetached(command, QStringList() << command_params << "--select" << url.toLocalFile());
   }
   else if (command.startsWith("dolphin") || command.startsWith("konqueror") || command.startsWith("kfmclient")) {
-    proc.startDetached(command, QStringList() << command_params << "--select" << "--new-window" << path);
+    proc.startDetached(command, QStringList() << command_params << "--select" << "--new-window" << url.toLocalFile());
   }
   else if (command.startsWith("caja")) {
-    QFileInfo info(path);
-    if (!info.exists()) return;
-    QString directory = info.dir().path();
-    proc.startDetached(command, QStringList() << command_params << "--no-desktop" << directory);
+    proc.startDetached(command, QStringList() << command_params << "--no-desktop" << path);
+  }
+  else if (command.startsWith("pcmanfm")) {
+    proc.startDetached(command, QStringList() << command_params << path);
   }
   else {
-    proc.startDetached(command, QStringList() << command_params << path);
+    proc.startDetached(command, QStringList() << command_params << url.toLocalFile());
   }
 
 }
@@ -393,27 +401,42 @@ void OpenInFileManager(const QString &path) {
 
 #ifdef Q_OS_MACOS
 // Better than openUrl(dirname(path)) - also highlights file at path
-void RevealFileInFinder(QString const &path) {
+void RevealFileInFinder(const QString &path) {
   QProcess::execute("/usr/bin/open", QStringList() << "-R" << path);
 }
 #endif  // Q_OS_MACOS
 
 #ifdef Q_OS_WIN
-void ShowFileInExplorer(QString const &path) {
+void ShowFileInExplorer(const QString &path);
+void ShowFileInExplorer(const QString &path) {
   QProcess::execute("explorer.exe", QStringList() << "/select," << QDir::toNativeSeparators(path));
 }
 #endif
 
 void OpenInFileBrowser(const QList<QUrl> &urls) {
 
-  if (urls.count() > 50) {
+  QMap<QString, QUrl> dirs;
+
+  for (const QUrl &url : urls) {
+    if (!url.isLocalFile()) {
+      continue;
+    }
+    QString path = url.toLocalFile();
+    if (!QFile::exists(path)) continue;
+
+    const QString directory = QFileInfo(path).dir().path();
+    if (dirs.contains(directory)) continue;
+    dirs.insert(directory, url);
+  }
+
+  if (dirs.count() > 50) {
     QMessageBox messagebox(QMessageBox::Critical, tr("Show in file browser"), tr("Too many songs selected."));
     messagebox.exec();
     return;
   }
 
-  if (urls.count() > 5) {
-    QMessageBox messagebox(QMessageBox::Information, tr("Show in file browser"), tr("%1 songs selected, are you sure you want to open them all?").arg(urls.count()), QMessageBox::Open|QMessageBox::Cancel);
+  if (dirs.count() > 5) {
+    QMessageBox messagebox(QMessageBox::Information, tr("Show in file browser"), tr("%1 songs in %2 different directories selected, are you sure you want to open them all?").arg(urls.count()).arg(dirs.count()), QMessageBox::Open|QMessageBox::Cancel);
     messagebox.setTextFormat(Qt::RichText);
     int result = messagebox.exec();
     switch (result) {
@@ -425,33 +448,21 @@ void OpenInFileBrowser(const QList<QUrl> &urls) {
     }
   }
 
-  QSet<QString> dirs;
-
-  for (const QUrl &url : urls) {
-    if (!url.isLocalFile()) {
-      continue;
-    }
-    QString path = url.toLocalFile();
-
-    if (!QFile::exists(path)) continue;
-
-    const QString directory = QFileInfo(path).dir().path();
-    if (dirs.contains(directory)) continue;
-    dirs.insert(directory);
-
+  QMap<QString, QUrl>::iterator i;
+  for (i = dirs.begin(); i != dirs.end(); ++i) {
 #if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
-    OpenInFileManager(path);
+    OpenInFileManager(i.key(), i.value());
 #elif defined(Q_OS_MACOS)
     // Revealing multiple files in the finder only opens one window, so it also makes sense to reveal at most one per directory
-    RevealFileInFinder(path);
+    RevealFileInFinder(i.value().toLocalFile());
 #elif defined(Q_OS_WIN32)
-    ShowFileInExplorer(path);
+    ShowFileInExplorer(i.value().toLocalFile());
 #endif
   }
 
 }
 
-QByteArray Hmac(const QByteArray &key, const QByteArray &data, HashFunction method) {
+QByteArray Hmac(const QByteArray &key, const QByteArray &data, const HashFunction method) {
 
   const int kBlockSize = 64;  // bytes
   Q_ASSERT(key.length() <= kBlockSize);
@@ -520,20 +531,14 @@ QString PrettySize(const QSize &size) {
   return QString::number(size.width()) + "x" + QString::number(size.height());
 }
 
-void ForwardMouseEvent(const QMouseEvent *e, QWidget *target) {
-  QMouseEvent c(e->type(), target->mapFromGlobal(e->globalPos()), e->globalPos(), e->button(), e->buttons(), e->modifiers());
-
-  QApplication::sendEvent(target, &c);
-}
-
-bool IsMouseEventInWidget(const QMouseEvent *e, const QWidget *widget) {
-  return widget->rect().contains(widget->mapFromGlobal(e->globalPos()));
-}
-
 quint16 PickUnusedPort() {
 
   forever {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+    const quint64 port = QRandomGenerator::global()->bounded(49152, 65535);
+#else
     const quint16 port = 49152 + qrand() % 16384;
+#endif
 
     QTcpServer server;
     if (server.listen(QHostAddress::Any, port)) {
@@ -596,8 +601,9 @@ bool ParseUntilElementCI(QXmlStreamReader *reader, const QString &name) {
 
 QDateTime ParseRFC822DateTime(const QString &text) {
 
-  QRegExp regexp("(\\d{1,2}) (\\w{3,12}) (\\d+) (\\d{1,2}):(\\d{1,2}):(\\d{1,2})");
-  if (regexp.indexIn(text) == -1) {
+  QRegularExpression regexp("(\\d{1,2}) (\\w{3,12}) (\\d+) (\\d{1,2}):(\\d{1,2}):(\\d{1,2})");
+  QRegularExpressionMatch re_match = regexp.match(text);
+  if (!re_match.hasMatch()) {
     return QDateTime();
   }
 
@@ -629,9 +635,9 @@ QDateTime ParseRFC822DateTime(const QString &text) {
   monthmap["November"] = 11;
   monthmap["December"] = 12;
 
-  const QDate date(regexp.cap(static_cast<int>(MatchNames::YEARS)).toInt(), monthmap[regexp.cap(static_cast<int>(MatchNames::MONTHS))], regexp.cap(static_cast<int>(MatchNames::DAYS)).toInt());
+  const QDate date(re_match.captured(static_cast<int>(MatchNames::YEARS)).toInt(), monthmap[re_match.captured(static_cast<int>(MatchNames::MONTHS))], re_match.captured(static_cast<int>(MatchNames::DAYS)).toInt());
 
-  const QTime time(regexp.cap(static_cast<int>(MatchNames::HOURS)).toInt(), regexp.cap(static_cast<int>(MatchNames::MINUTES)).toInt(), regexp.cap(static_cast<int>(MatchNames::SECONDS)).toInt());
+  const QTime time(re_match.captured(static_cast<int>(MatchNames::HOURS)).toInt(), re_match.captured(static_cast<int>(MatchNames::MINUTES)).toInt(), re_match.captured(static_cast<int>(MatchNames::SECONDS)).toInt());
 
   return QDateTime(date, time);
 
@@ -643,7 +649,7 @@ const char *EnumToString(const QMetaObject &meta, const char *name, int value) {
   if (index == -1) return "[UnknownEnum]";
   QMetaEnum metaenum = meta.enumerator(index);
   const char *result = metaenum.valueToKey(value);
-  if (result == 0) return "[UnknownEnumValue]";
+  if (!result) return "[UnknownEnumValue]";
   return result;
 
 }
@@ -814,8 +820,12 @@ QString CryptographicRandomString(const int len) {
 QString GetRandomString(const int len, const QString &UseCharacters) {
 
    QString randstr;
-   for(int i=0 ; i < len ; ++i) {
-     int index = qrand() % UseCharacters.length();
+   for(int i = 0 ; i < len ; ++i) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+     const int index = QRandomGenerator::global()->bounded(0, UseCharacters.length());
+#else
+     const int index = qrand() % UseCharacters.length();
+#endif
      QChar nextchar = UseCharacters.at(index);
      randstr.append(nextchar);
    }
@@ -857,7 +867,7 @@ QString UnicodeToAscii(const QString &unicode) {
 #endif
 
   iconv_t conv = iconv_open("ASCII//TRANSLIT", "UTF-8");
-  if (conv == (iconv_t) -1) return unicode;
+  if (conv == reinterpret_cast<iconv_t>(-1)) return unicode;
 
   QByteArray utf8 = unicode.toUtf8();
 
@@ -911,18 +921,22 @@ QString MacAddress() {
 
 QString ReplaceMessage(const QString &message, const Song &song, const QString &newline) {
 
-  QRegExp variable_replacer("[%][a-z]+[%]");
+  QRegularExpression variable_replacer("[%][a-z]+[%]");
   QString copy(message);
 
   // Replace the first line
   int pos = 0;
-  variable_replacer.indexIn(message);
-  while ((pos = variable_replacer.indexIn(message, pos)) != -1) {
-    QStringList captured = variable_replacer.capturedTexts();
+  QRegularExpressionMatch match;
+  for (match = variable_replacer.match(message, pos) ; match.hasMatch() ; match = variable_replacer.match(message, pos)) {
+    pos = match.capturedStart();
+    QStringList captured = match.capturedTexts();
     copy.replace(captured[0], ReplaceVariable(captured[0], song, newline));
-    pos += variable_replacer.matchedLength();
+    pos += match.capturedLength();
   }
 
+  int index_of = copy.indexOf(QRegularExpression(" - (>|$)"));
+  if (index_of >= 0) copy = copy.remove(index_of, 3);
+
   return copy;
 }
 
@@ -982,6 +996,20 @@ QString ReplaceVariable(const QString &variable, const Song &song, const QString
   return variable;
 }
 
+bool IsColorDark(const QColor &color) {
+  return ((30 * color.red() + 59 * color.green() + 11 * color.blue()) / 100) <= 130;
+}
+
+QList<QByteArray> ImageFormatsForMimeType(const QByteArray &mimetype) {
+
+  if (mimetype == "image/bmp") return QList<QByteArray>() << "BMP";
+  else if (mimetype == "image/gif") return QList<QByteArray>() << "GIF";
+  else if (mimetype == "image/jpeg") return QList<QByteArray>() << "JPG";
+  else if (mimetype == "image/png") return QList<QByteArray>() << "PNG";
+  else return QList<QByteArray>();
+
+}
+
 }  // namespace Utilities
 
 ScopedWCharArray::ScopedWCharArray(const QString &str)
