@@ -19,13 +19,17 @@
 
 #include "config.h"
 
+#include <algorithm>
+
 #include <QtGlobal>
 #include <QObject>
+#include <QQueue>
 #include <QVariant>
 #include <QByteArray>
 #include <QString>
 #include <QUrl>
 #include <QUrlQuery>
+#include <QTimer>
 #include <QNetworkAccessManager>
 #include <QNetworkRequest>
 #include <QNetworkReply>
@@ -36,22 +40,56 @@
 #include <QtDebug>
 
 #include "core/application.h"
-#include "core/closure.h"
 #include "core/network.h"
 #include "core/logging.h"
 #include "albumcoverfetcher.h"
-#include "coverprovider.h"
+#include "jsoncoverprovider.h"
 #include "musicbrainzcoverprovider.h"
 
 const char *MusicbrainzCoverProvider::kReleaseSearchUrl = "https://musicbrainz.org/ws/2/release/";
 const char *MusicbrainzCoverProvider::kAlbumCoverUrl = "https://coverartarchive.org/release/%1/front";
 const int MusicbrainzCoverProvider::kLimit = 8;
+const int MusicbrainzCoverProvider::kRequestsDelay = 1000;
 
-MusicbrainzCoverProvider::MusicbrainzCoverProvider(Application *app, QObject *parent): CoverProvider("MusicBrainz", 1.5, true, app, parent), network_(new NetworkAccessManager(this)) {}
+MusicbrainzCoverProvider::MusicbrainzCoverProvider(Application *app, QObject *parent): JsonCoverProvider("MusicBrainz", true, false, 1.5, true, false, app, parent), network_(new NetworkAccessManager(this)), timer_flush_requests_(new QTimer(this)) {
 
-bool MusicbrainzCoverProvider::StartSearch(const QString &artist, const QString &album, const int id) {
+  timer_flush_requests_->setInterval(kRequestsDelay);
+  timer_flush_requests_->setSingleShot(false);
+  connect(timer_flush_requests_, SIGNAL(timeout()), this, SLOT(FlushRequests()));
 
-  QString query = QString("release:\"%1\" AND artist:\"%2\"").arg(album.trimmed().replace('"', "\\\"")).arg(artist.trimmed().replace('"', "\\\""));
+}
+
+MusicbrainzCoverProvider::~MusicbrainzCoverProvider() {
+
+  while (!replies_.isEmpty()) {
+    QNetworkReply *reply = replies_.takeFirst();
+    disconnect(reply, nullptr, this, nullptr);
+    reply->abort();
+    reply->deleteLater();
+  }
+
+}
+
+bool MusicbrainzCoverProvider::StartSearch(const QString &artist, const QString &album, const QString &title, const int id) {
+
+  Q_UNUSED(title);
+
+  if (artist.isEmpty() || album.isEmpty()) return false;
+
+  SearchRequest request(id, artist, album);
+  queue_search_requests_ << request;
+
+  if (!timer_flush_requests_->isActive()) {
+    timer_flush_requests_->start();
+  }
+
+  return true;
+
+}
+
+void MusicbrainzCoverProvider::SendSearchRequest(const SearchRequest &request) {
+
+  QString query = QString("release:\"%1\" AND artist:\"%2\"").arg(request.album.trimmed().replace('"', "\\\"")).arg(request.artist.trimmed().replace('"', "\\\""));
 
   QUrlQuery url_query;
   url_query.addQueryItem("query", query);
@@ -61,18 +99,33 @@ bool MusicbrainzCoverProvider::StartSearch(const QString &artist, const QString
   QUrl url(kReleaseSearchUrl);
   url.setQuery(url_query);
   QNetworkRequest req(url);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
+  req.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
+#else
   req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
+#endif
   QNetworkReply *reply = network_->get(req);
-  NewClosure(reply, SIGNAL(finished()), this, SLOT(HandleSearchReply(QNetworkReply *, int)), reply, id);
-
-  return true;
+  replies_ << reply;
+  connect(reply, &QNetworkReply::finished, [=] { HandleSearchReply(reply, request.id); });
 
 }
 
-void MusicbrainzCoverProvider::CancelSearch(const int id) { Q_UNUSED(id); }
+void MusicbrainzCoverProvider::FlushRequests() {
+
+  if (!queue_search_requests_.isEmpty()) {
+    SendSearchRequest(queue_search_requests_.dequeue());
+    return;
+  }
+
+  timer_flush_requests_->stop();
+
+}
 
 void MusicbrainzCoverProvider::HandleSearchReply(QNetworkReply *reply, const int search_id) {
 
+  if (!replies_.contains(reply)) return;
+  replies_.removeAll(reply);
+  disconnect(reply, nullptr, this, nullptr);
   reply->deleteLater();
 
   CoverSearchResults results;
@@ -100,69 +153,70 @@ void MusicbrainzCoverProvider::HandleSearchReply(QNetworkReply *reply, const int
     emit SearchFinished(search_id, results);
     return;
   }
-  QJsonValue json_value = json_obj["releases"];
+  QJsonValue value_releases = json_obj["releases"];
 
-  if (!json_value.isArray()) {
-    Error("Json releases is not an array.", json_value);
+  if (!value_releases.isArray()) {
+    Error("Json releases is not an array.", value_releases);
     emit SearchFinished(search_id, results);
     return;
   }
-  QJsonArray json_releases = json_value.toArray();
+  QJsonArray array_releases = value_releases.toArray();
 
-  if (json_releases.isEmpty()) {
+  if (array_releases.isEmpty()) {
     emit SearchFinished(search_id, results);
     return;
   }
 
-  for (const QJsonValue &value : json_releases) {
+  for (const QJsonValue &value_release : array_releases) {
 
-    if (!value.isObject()) {
-      Error("Invalid Json reply, album value is not an object.", value);
+    if (!value_release.isObject()) {
+      Error("Invalid Json reply, releases array value is not an object.", value_release);
       continue;
     }
-    QJsonObject json_obj = value.toObject();
-    if (!json_obj.contains("id") || !json_obj.contains("artist-credit") ||  !json_obj.contains("title")) {
-      Error("Invalid Json reply, album is missing id, artist-credit or title.", json_obj);
+    QJsonObject obj_release = value_release.toObject();
+    if (!obj_release.contains("id") || !obj_release.contains("artist-credit") ||  !obj_release.contains("title")) {
+      Error("Invalid Json reply, releases array object is missing id, artist-credit or title.", obj_release);
       continue;
     }
 
-    QJsonValue json_artists = json_obj["artist-credit"];
+    QJsonValue json_artists = obj_release["artist-credit"];
     if (!json_artists.isArray()) {
-      Error("Json artist-credit is not an array.", json_artists);
+      Error("Invalid Json reply, artist-credit is not a array.", json_artists);
       continue;
     }
-    QJsonArray json_array_artists = json_artists.toArray();
+    QJsonArray array_artists = json_artists.toArray();
     int i = 0;
     QString artist;
-    for (const QJsonValue &json_value_artist : json_array_artists) {
-      if (!json_value_artist.isObject()) {
-        Error("Invalid Json reply, artist is not an object.", json_value_artist);
+    for (const QJsonValue &value_artist : array_artists) {
+      if (!value_artist.isObject()) {
+        Error("Invalid Json reply, artist is not a object.", value_artist);
         continue;
       }
-      QJsonObject json_obj_artist = json_value_artist.toObject();
+      QJsonObject obj_artist = value_artist.toObject();
 
-      if (!json_obj_artist.contains("artist") ) {
-        Error("Invalid Json reply, artist is missing.", json_obj_artist);
+      if (!obj_artist.contains("artist") ) {
+        Error("Invalid Json reply, artist is missing.", obj_artist);
         continue;
       }
-      QJsonValue json_value_artist2 = json_obj_artist["artist"];
-      if (!json_value_artist2.isObject()) {
-        Error("Invalid Json reply, artist is not an object.", json_value_artist2);
+      QJsonValue value_artist2 = obj_artist["artist"];
+      if (!value_artist2.isObject()) {
+        Error("Invalid Json reply, artist is not an object.", value_artist2);
         continue;
       }
-      QJsonObject json_obj_artist2 = json_value_artist2.toObject();
+      QJsonObject obj_artist2 = value_artist2.toObject();
 
-      if (!json_obj_artist2.contains("name") ) {
-        Error("Invalid Json reply, artist is missing name.", json_value_artist2);
+      if (!obj_artist2.contains("name") ) {
+        Error("Invalid Json reply, artist is missing name.", value_artist2);
         continue;
       }
-      artist = json_obj_artist2["name"].toString();
+      artist = obj_artist2["name"].toString();
       ++i;
     }
     if (i > 1) artist = "Various artists";
 
-    QString id = json_obj["id"].toString();
-    QString album = json_obj["title"].toString();
+    QString id = obj_release["id"].toString();
+    QString album = obj_release["title"].toString();
+
     CoverSearchResult cover_result;
     QUrl url(QString(kAlbumCoverUrl).arg(id));
     cover_result.artist = artist;
@@ -216,34 +270,9 @@ QByteArray MusicbrainzCoverProvider::GetReplyData(QNetworkReply *reply) {
 
 }
 
-QJsonObject MusicbrainzCoverProvider::ExtractJsonObj(const QByteArray &data) {
+void MusicbrainzCoverProvider::Error(const QString &error, const QVariant &debug) {
 
-  QJsonParseError error;
-  QJsonDocument json_doc = QJsonDocument::fromJson(data, &error);
-
-  if (error.error != QJsonParseError::NoError) {
-    Error("Reply from server is missing Json data.", data);
-    return QJsonObject();
-  }
-  if (json_doc.isNull() || json_doc.isEmpty()) {
-    Error("Received empty Json document.", json_doc);
-    return QJsonObject();
-  }
-  if (!json_doc.isObject()) {
-    Error("Json document is not an object.", json_doc);
-    return QJsonObject();
-  }
-  QJsonObject json_obj = json_doc.object();
-  if (json_obj.isEmpty()) {
-    Error("Received empty Json object.", json_doc);
-    return QJsonObject();
-  }
-
-  return json_obj;
-
-}
-
-void MusicbrainzCoverProvider::Error(QString error, QVariant debug) {
   qLog(Error) << "Musicbrainz:" << error;
   if (debug.isValid()) qLog(Debug) << debug;
+
 }
