@@ -38,6 +38,7 @@
 #include "core/tagreaderclient.h"
 #include "playlist/playlistitem.h"
 #include "covermanager/albumcoverloaderoptions.h"
+#include "covermanager/albumcoverloaderresult.h"
 
 class QWidget;
 class QMenu;
@@ -61,7 +62,7 @@ class EditTagDialog : public QDialog {
 
  public:
   explicit EditTagDialog(Application *app, QWidget *parent = nullptr);
-  ~EditTagDialog();
+  ~EditTagDialog() override;
 
   static const char *kHintText;
   static const char *kSettingsGroup;
@@ -70,19 +71,19 @@ class EditTagDialog : public QDialog {
 
   PlaylistItemList playlist_items() const { return playlist_items_; }
 
-  void accept();
+  void accept() override;
 
  signals:
   void Error(const QString &message);
 
  protected:
-  bool eventFilter(QObject *o, QEvent *e);
-  void showEvent(QShowEvent*);
-  void hideEvent(QHideEvent*);
+  bool eventFilter(QObject *o, QEvent *e) override;
+  void showEvent(QShowEvent*) override;
+  void hideEvent(QHideEvent*) override;
 
  private:
   struct Data {
-    Data(const Song &song = Song()) : original_(song), current_(song) {}
+    explicit Data(const Song &song = Song()) : original_(song), current_(song) {}
 
     static QVariant value(const Song &song, const QString &id);
     QVariant original_value(const QString &id) const {
@@ -110,7 +111,7 @@ class EditTagDialog : public QDialog {
   void FetchTag();
   void FetchTagSongChosen(const Song &original_song, const Song &new_metadata);
 
-  void AlbumCoverLoaded(const quint64 id, const QUrl &cover_url, const QImage &scaled, const QImage &original);
+  void AlbumCoverLoaded(const quint64 id, const AlbumCoverLoaderResult &result);
 
   void LoadCoverFromFile();
   void SaveCoverToFile();
@@ -126,7 +127,7 @@ class EditTagDialog : public QDialog {
 
  private:
   struct FieldData {
-    FieldData(QLabel *label = nullptr, QWidget *editor = nullptr, const QString &id = QString())
+    explicit FieldData(QLabel *label = nullptr, QWidget *editor = nullptr, const QString &id = QString())
         : label_(label), editor_(editor), id_(id) {}
 
     QLabel *label_;
@@ -155,7 +156,7 @@ class EditTagDialog : public QDialog {
 
   // Called by QtConcurrentRun
   QList<Data> LoadData(const SongList &songs) const;
-  void SaveData(const QList<Data> &data);
+  void SaveData(const QList<Data> &tag_data);
 
  private:
   Ui_EditTagDialog *ui_;
