@@ -35,7 +35,7 @@
 #include "collection/sqlrow.h"
 #include "playlist/playlistbackend.h"
 
-InternetPlaylistItem::InternetPlaylistItem(const Song::Source &source)
+InternetPlaylistItem::InternetPlaylistItem(const Song::Source source)
     : PlaylistItem(source) {}
 
 InternetPlaylistItem::InternetPlaylistItem(InternetService *service, const Song &metadata)
@@ -46,9 +46,11 @@ InternetPlaylistItem::InternetPlaylistItem(InternetService *service, const Song
 }
 
 bool InternetPlaylistItem::InitFromQuery(const SqlRow &query) {
+
   metadata_.InitFromQuery(query, false, (Song::kColumns.count() + 1) * 1);
   InitMetadata();
   return true;
+
 }
 
 QVariant InternetPlaylistItem::DatabaseValue(DatabaseColumn column) const {
@@ -56,15 +58,26 @@ QVariant InternetPlaylistItem::DatabaseValue(DatabaseColumn column) const {
 }
 
 void InternetPlaylistItem::InitMetadata() {
+
   if (metadata_.title().isEmpty()) metadata_.set_title(metadata_.url().toString());
   if (metadata_.source() == Song::Source_Unknown) metadata_.set_source(Song::Source_Stream);
   if (metadata_.filetype() == Song::FileType_Unknown) metadata_.set_filetype(Song::FileType_Stream);
   metadata_.set_valid(true);
+
 }
 
 Song InternetPlaylistItem::Metadata() const {
+
   if (HasTemporaryMetadata()) return temp_metadata_;
   return metadata_;
+
 }
 
 QUrl InternetPlaylistItem::Url() const { return metadata_.url(); }
+
+void InternetPlaylistItem::SetArtManual(const QUrl &cover_url) {
+
+  metadata_.set_art_manual(cover_url);
+  temp_metadata_.set_art_manual(cover_url);
+
+}
