@@ -21,7 +21,7 @@
 
 #include "config.h"
 
-#include <stdlib.h>
+#include <cstdlib>
 #include <memory>
 #include <utility>
 #include <algorithm>
@@ -33,7 +33,7 @@
 #include <QtGlobal>
 #include <QObject>
 #include <QCoreApplication>
-#include <QtConcurrentRun>
+#include <QtConcurrent>
 #include <QFuture>
 #include <QIODevice>
 #include <QDataStream>
@@ -69,6 +69,7 @@
 #include "collection/collection.h"
 #include "collection/collectionbackend.h"
 #include "collection/collectionplaylistitem.h"
+#include "covermanager/albumcoverloader.h"
 #include "queue/queue.h"
 #include "playlist.h"
 #include "playlistitem.h"
@@ -88,12 +89,6 @@
 
 using std::placeholders::_1;
 using std::placeholders::_2;
-using std::shared_ptr;
-using std::unordered_map;
-using std::sort;
-using std::stable_sort;
-using std::greater;
-using std::swap;
 
 const char *Playlist::kCddaMimeType = "x-content/audio-cdda";
 const char *Playlist::kRowsMimetype = "application/x-strawberry-playlist-rows";
@@ -141,8 +136,8 @@ Playlist::Playlist(PlaylistBackend *backend, TaskManager *task_manager, Collecti
 
   undo_stack_->setUndoLimit(kUndoStackSize);
 
-  connect(this, SIGNAL(rowsInserted(const QModelIndex&, int, int)), SIGNAL(PlaylistChanged()));
-  connect(this, SIGNAL(rowsRemoved(const QModelIndex&, int, int)), SIGNAL(PlaylistChanged()));
+  connect(this, SIGNAL(rowsInserted(QModelIndex, int, int)), SIGNAL(PlaylistChanged()));
+  connect(this, SIGNAL(rowsRemoved(QModelIndex, int, int)), SIGNAL(PlaylistChanged()));
 
   Restore();
 
@@ -152,7 +147,7 @@ Playlist::Playlist(PlaylistBackend *backend, TaskManager *task_manager, Collecti
   connect(queue_, SIGNAL(rowsAboutToBeRemoved(QModelIndex, int, int)), SLOT(TracksAboutToBeDequeued(QModelIndex, int, int)));
   connect(queue_, SIGNAL(rowsRemoved(QModelIndex,int,int)), SLOT(TracksDequeued()));
 
-  connect(queue_, SIGNAL(rowsInserted(const QModelIndex&, int, int)), SLOT(TracksEnqueued(const QModelIndex&, int, int)));
+  connect(queue_, SIGNAL(rowsInserted(QModelIndex, int, int)), SLOT(TracksEnqueued(QModelIndex, int, int)));
 
   connect(queue_, SIGNAL(layoutChanged()), SLOT(QueueLayoutChanged()));
 
@@ -182,7 +177,7 @@ QVariant Playlist::headerData(int section, Qt::Orientation, int role) const {
 
   if (role != Qt::DisplayRole && role != Qt::ToolTipRole) return QVariant();
 
-  const QString name = column_name((Playlist::Column)section);
+  const QString name = column_name(static_cast<Playlist::Column>(section));
   if (!name.isEmpty()) return name;
 
   return QVariant();
@@ -256,6 +251,7 @@ bool Playlist::set_column_value(Song &song, Playlist::Column column, const QVari
     default:
       break;
   }
+
   return true;
 
 }
@@ -375,11 +371,11 @@ bool Playlist::setData(const QModelIndex &index, const QVariant &value, int role
 
   int row = index.row();
   PlaylistItemPtr item = item_at(row);
-  Song song = item->Metadata();
+  Song song = item->OriginalMetadata();
 
   if (index.data() == value) return false;
 
-  if (!set_column_value(song, (Column)index.column(), value)) return false;
+  if (!set_column_value(song, static_cast<Column>(index.column()), value)) return false;
 
   TagReaderReply *reply = TagReaderClient::Instance()->SaveFile(song.url().toLocalFile(), song);
   NewClosure(reply, SIGNAL(Finished(bool)), this, SLOT(SongSaveComplete(TagReaderReply*, QPersistentModelIndex)), reply, QPersistentModelIndex(index));
@@ -392,7 +388,9 @@ void Playlist::SongSaveComplete(TagReaderReply *reply, const QPersistentModelInd
 
   if (reply->is_successful() && index.isValid()) {
     if (reply->message().save_file_response().success()) {
-      QFuture<void> future = item_at(index.row())->BackgroundReload();
+      PlaylistItemPtr item = item_at(index.row());
+      if (!item) return;
+      QFuture<void> future = item->BackgroundReload();
       NewClosure(future, this, SLOT(ItemReloadComplete(QPersistentModelIndex)), index);
     }
     else {
@@ -406,6 +404,12 @@ void Playlist::SongSaveComplete(TagReaderReply *reply, const QPersistentModelInd
 void Playlist::ItemReloadComplete(const QPersistentModelIndex &index) {
 
   if (index.isValid()) {
+
+    PlaylistItemPtr item = item_at(index.row());
+    if (item && item->HasTemporaryMetadata()) {  // Update temporary metadata.
+      item->UpdateTemporaryMetadata(item->OriginalMetadata());
+    }
+
     emit dataChanged(index, index);
     emit EditingFinished(index);
   }
@@ -467,7 +471,7 @@ int Playlist::NextVirtualIndex(int i, bool ignore_repeat_track) const {
     }
     Song this_song = item_at(virtual_items_[j])->Metadata();
     if (((last_song.is_compilation() && this_song.is_compilation()) ||
-         last_song.artist() == this_song.artist()) &&
+         last_song.effective_albumartist() == this_song.effective_albumartist()) &&
         last_song.album() == this_song.album() &&
         FilterContainsVirtualIndex(j)) {
       return j;  // Found one
@@ -599,7 +603,6 @@ void Playlist::set_current_row(int i, bool is_stopping) {
 
   if (current_item_index_ == old_current_item_index) {
     UpdateScrobblePoint();
-    nowplaying_ = false;
     return;
   }
 
@@ -637,7 +640,6 @@ void Playlist::set_current_row(int i, bool is_stopping) {
   }
 
   UpdateScrobblePoint();
-  nowplaying_ = false;
 
 }
 
@@ -645,7 +647,7 @@ Qt::ItemFlags Playlist::flags(const QModelIndex &index) const {
 
   Qt::ItemFlags flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 
-  if (column_is_editable((Column)index.column())) flags |= Qt::ItemIsEditable;
+  if (column_is_editable(static_cast<Column>(index.column()))) flags |= Qt::ItemIsEditable;
 
   if (index.isValid()) return flags | Qt::ItemIsDragEnabled;
 
@@ -708,7 +710,7 @@ bool Playlist::dropMimeData(const QMimeData *data, Qt::DropAction action, int ro
     stream.readRawData(reinterpret_cast<char*>(&source_playlist), sizeof(source_playlist));
     stream >> source_rows;
     if (!stream.atEnd()) {
-      stream.readRawData((char*)&pid, sizeof(pid));
+      stream.readRawData(reinterpret_cast<char*>(&pid), sizeof(pid));
     }
     else {
       pid = !own_pid;
@@ -723,7 +725,7 @@ bool Playlist::dropMimeData(const QMimeData *data, Qt::DropAction action, int ro
     else if (pid == own_pid) {
       // Drag from a different playlist
       PlaylistItemList items;
-      for (int row : source_rows) items << source_playlist->item_at(row);
+      for (const int i : source_rows) items << source_playlist->item_at(i);
 
       if (items.count() > kUndoItemLimit) {
         // Too big to keep in the undo stack. Also clear the stack because it might have been invalidated.
@@ -736,8 +738,8 @@ bool Playlist::dropMimeData(const QMimeData *data, Qt::DropAction action, int ro
 
       // Remove the items from the source playlist if it was a move event
       if (action == Qt::MoveAction) {
-        for (int row : source_rows) {
-          source_playlist->undo_stack()->push(new PlaylistUndoCommands::RemoveItems(source_playlist, row, 1));
+        for (const int i : source_rows) {
+          source_playlist->undo_stack()->push(new PlaylistUndoCommands::RemoveItems(source_playlist, i, 1));
         }
       }
     }
@@ -879,7 +881,7 @@ void Playlist::InsertItems(const PlaylistItemList &itemsIn, int pos, bool play_n
   // exercise vetoes
   SongList songs;
 
-  for (const PlaylistItemPtr item : items) {
+  for (PlaylistItemPtr item : items) {
     songs << item->Metadata();
   }
 
@@ -920,7 +922,8 @@ void Playlist::InsertItems(const PlaylistItemList &itemsIn, int pos, bool play_n
     // Too big to keep in the undo stack. Also clear the stack because it might have been invalidated.
     InsertItemsWithoutUndo(items, pos, enqueue, enqueue_next);
     undo_stack_->clear();
-  } else {
+  }
+  else {
     undo_stack_->push(new PlaylistUndoCommands::InsertItems(this, items, pos, enqueue, enqueue_next));
   }
 
@@ -944,7 +947,7 @@ void Playlist::InsertItemsWithoutUndo(const PlaylistItemList &items, int pos, bo
     if (item->source() == Song::Source_Collection) {
       int id = item->Metadata().id();
       if (id != -1) {
-        collection_items_by_id_.insertMulti(id, item);
+        collection_items_by_id_.insert(id, item);
       }
     }
 
@@ -977,15 +980,15 @@ void Playlist::InsertItemsWithoutUndo(const PlaylistItemList &items, int pos, bo
 
 }
 
-void Playlist::InsertCollectionItems(const SongList &songs, int pos, bool play_now, bool enqueue, bool enqueue_next) {
+void Playlist::InsertCollectionItems(const SongList &songs, const int pos, const bool play_now, const bool enqueue, const bool enqueue_next) {
   InsertSongItems<CollectionPlaylistItem>(songs, pos, play_now, enqueue, enqueue_next);
 }
 
-void Playlist::InsertSongs(const SongList &songs, int pos, bool play_now, bool enqueue, bool enqueue_next) {
+void Playlist::InsertSongs(const SongList &songs, const int pos, const bool play_now, const bool enqueue, const bool enqueue_next) {
   InsertSongItems<SongPlaylistItem>(songs, pos, play_now, enqueue, enqueue_next);
 }
 
-void Playlist::InsertSongsOrCollectionItems(const SongList &songs, int pos, bool play_now, bool enqueue, bool enqueue_next) {
+void Playlist::InsertSongsOrCollectionItems(const SongList &songs, const int pos, const bool play_now, const bool enqueue, const bool enqueue_next) {
 
   PlaylistItemList items;
   for (const Song &song : songs) {
@@ -1000,48 +1003,39 @@ void Playlist::InsertSongsOrCollectionItems(const SongList &songs, int pos, bool
 
 }
 
-void Playlist::InsertInternetItems(InternetService *service, const SongList &songs, int pos, bool play_now, bool enqueue, bool enqueue_next) {
+void Playlist::InsertInternetItems(InternetService *service, const SongList &songs, const int pos, const bool play_now, const bool enqueue, const bool enqueue_next) {
 
   PlaylistItemList playlist_items;
   for (const Song &song : songs) {
-    playlist_items << shared_ptr<PlaylistItem>(new InternetPlaylistItem(service, song));
+    playlist_items << std::shared_ptr<PlaylistItem>(new InternetPlaylistItem(service, song));
   }
 
   InsertItems(playlist_items, pos, play_now, enqueue, enqueue_next);
 
 }
 
-void Playlist::UpdateItems(const SongList &songs) {
+void Playlist::UpdateItems(SongList songs) {
 
   qLog(Debug) << "Updating playlist with new tracks' info";
+
   // We first convert our songs list into a linked list (a 'real' list), because removals are faster with QLinkedList.
   // Next, we walk through the list of playlist's items then the list of songs
   // we want to update: if an item corresponds to the song (we rely on URL for this), we update the item with the new metadata,
   // then we remove song from our list because we will not need to check it again.
   // And we also update undo actions.
-  QLinkedList<Song> songs_list;
-  for (const Song &song : songs) songs_list.append(song);
 
-  for (int i = 0; i < items_.size(); i++) {
+  for (int i = 0;  i < items_.size() ; i++) {
     // Update current items list
-    QMutableLinkedListIterator<Song> it(songs_list);
+    QMutableListIterator<Song> it(songs);
     while (it.hasNext()) {
       const Song &song = it.next();
-      PlaylistItemPtr &item = items_[i];
-      if (item->Metadata().url() == song.url() &&
-          (
-           item->Metadata().source() == Song::Source_Unknown ||
-           item->Metadata().filetype() == Song::FileType_Unknown ||
-           // Stream may change and may need to be updated too
-           item->Metadata().is_stream() ||
-           // And CD tracks as well (tags are loaded in a second step)
-           item->Metadata().is_cdda()
-          )
-       ) {
+      const PlaylistItemPtr &item = items_[i];
+      if (item->Metadata().url() == song.url() && (item->Metadata().filetype() == Song::FileType_Unknown || item->Metadata().filetype() == Song::FileType_Stream || item->Metadata().filetype() == Song::FileType_CDDA)) {
         PlaylistItemPtr new_item;
         if (song.is_collection_song()) {
           new_item = PlaylistItemPtr(new CollectionPlaylistItem(song));
-          collection_items_by_id_.insertMulti(song.id(), new_item);
+          if (collection_items_by_id_.contains(song.id(), item)) collection_items_by_id_.remove(song.id(), item);
+          collection_items_by_id_.insert(song.id(), new_item);
         }
         else {
           new_item = PlaylistItemPtr(new SongPlaylistItem(song));
@@ -1049,7 +1043,7 @@ void Playlist::UpdateItems(const SongList &songs) {
         items_[i] = new_item;
         emit dataChanged(index(i, 0), index(i, ColumnCount - 1));
         // Also update undo actions
-        for (int i = 0; i < undo_stack_->count(); i++) {
+        for (int y = 0 ; y < undo_stack_->count() ; y++) {
           QUndoCommand *undo_action = const_cast<QUndoCommand*>(undo_stack_->command(i));
           PlaylistUndoCommands::InsertItems *undo_action_insert = dynamic_cast<PlaylistUndoCommands::InsertItems*>(undo_action);
           if (undo_action_insert) {
@@ -1073,7 +1067,7 @@ QMimeData *Playlist::mimeData(const QModelIndexList &indexes) const {
   // We only want one index per row, but we can't just take column 0 because the user might have hidden it.
   const int first_column = indexes.first().column();
 
-  QMimeData *data = new QMimeData;
+  QMimeData *mimedata = new QMimeData;
 
   QList<QUrl> urls;
   QList<int> rows;
@@ -1093,20 +1087,20 @@ QMimeData *Playlist::mimeData(const QModelIndexList &indexes) const {
 
   stream.writeRawData(reinterpret_cast<char*>(&self), sizeof(self));
   stream << rows;
-  stream.writeRawData((char*)&pid, sizeof(pid));
+  stream.writeRawData(reinterpret_cast<const char*>(&pid), sizeof(pid));
   buf.close();
 
-  data->setUrls(urls);
-  data->setData(kRowsMimetype, buf.data());
+  mimedata->setUrls(urls);
+  mimedata->setData(kRowsMimetype, buf.data());
 
-  return data;
+  return mimedata;
 
 }
 
-bool Playlist::CompareItems(int column, Qt::SortOrder order, shared_ptr<PlaylistItem> _a, shared_ptr<PlaylistItem> _b) {
+bool Playlist::CompareItems(int column, Qt::SortOrder order, std::shared_ptr<PlaylistItem> _a, std::shared_ptr<PlaylistItem> _b) {
 
-  shared_ptr<PlaylistItem> a = order == Qt::AscendingOrder ? _a : _b;
-  shared_ptr<PlaylistItem> b = order == Qt::AscendingOrder ? _b : _a;
+  std::shared_ptr<PlaylistItem> a = order == Qt::AscendingOrder ? _a : _b;
+  std::shared_ptr<PlaylistItem> b = order == Qt::AscendingOrder ? _b : _a;
 
 #define cmp(field) return a->Metadata().field() < b->Metadata().field()
 #define strcmp(field) return QString::localeAwareCompare(a->Metadata().field().toLower(), b->Metadata().field().toLower()) < 0;
@@ -1154,10 +1148,10 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order, shared_ptr<Playlist
 
 }
 
-bool Playlist::ComparePathDepths(Qt::SortOrder order, shared_ptr<PlaylistItem> _a, shared_ptr<PlaylistItem> _b) {
+bool Playlist::ComparePathDepths(Qt::SortOrder order, std::shared_ptr<PlaylistItem> _a, std::shared_ptr<PlaylistItem> _b) {
 
-  shared_ptr<PlaylistItem> a = order == Qt::AscendingOrder ? _a : _b;
-  shared_ptr<PlaylistItem> b = order == Qt::AscendingOrder ? _b : _a;
+  std::shared_ptr<PlaylistItem> a = order == Qt::AscendingOrder ? _a : _b;
+  std::shared_ptr<PlaylistItem> b = order == Qt::AscendingOrder ? _b : _a;
 
   int a_dir_level = a->Url().path().count('/');
   int b_dir_level = b->Url().path().count('/');
@@ -1293,6 +1287,7 @@ void Playlist::SetCurrentIsPaused(bool paused) {
 }
 
 void Playlist::Save() const {
+
   if (!backend_ || is_loading_) return;
 
   backend_->SavePlaylistAsync(id_, items_, last_played_row());
@@ -1308,7 +1303,7 @@ void Playlist::Restore() {
   collection_items_by_id_.clear();
 
   cancel_restore_ = false;
-  QFuture<QList<PlaylistItemPtr>> future = QtConcurrent::run(backend_, &PlaylistBackend::GetPlaylistItems, id_);
+  QFuture<QList<PlaylistItemPtr>> future = QtConcurrent::run(std::bind(&PlaylistBackend::GetPlaylistItems, backend_, id_));
   NewClosure(future, this, SLOT(ItemsLoaded(QFuture<PlaylistItemList>)), future);
 
 }
@@ -1347,7 +1342,7 @@ void Playlist::ItemsLoaded(QFuture<PlaylistItemList> future) {
 
   // Should we gray out deleted songs asynchronously on startup?
   if (greyout) {
-    QtConcurrent::run(this, &Playlist::InvalidateDeletedSongs);
+    (void)QtConcurrent::run(std::bind(&Playlist::InvalidateDeletedSongs, this));
   }
 
   emit PlaylistLoaded();
@@ -1379,6 +1374,8 @@ void Playlist::RemoveItemsWithoutUndo(const QList<int> &indicesIn) {
 
 bool Playlist::removeRows(int row, int count, const QModelIndex &parent) {
 
+  Q_UNUSED(parent);
+
   if (row < 0 || row >= items_.size() || row + count > items_.size()) {
     return false;
   }
@@ -1388,9 +1385,6 @@ bool Playlist::removeRows(int row, int count, const QModelIndex &parent) {
     RemoveItemsWithoutUndo(row, count);
     undo_stack_->clear();
   }
-  else if (parent == QModelIndex()) {
-    RemoveItemsWithoutUndo(row, count);
-  }
   else {
     undo_stack_->push(new PlaylistUndoCommands::RemoveItems(this, row, count));
   }
@@ -1428,7 +1422,7 @@ bool Playlist::removeRows(QList<int> &rows) {
 
 }
 
-PlaylistItemList Playlist::RemoveItemsWithoutUndo(int row, int count) {
+PlaylistItemList Playlist::RemoveItemsWithoutUndo(const int row, const int count) {
 
   if (row < 0 || row >= items_.size() || row + count > items_.size()) {
     return PlaylistItemList();
@@ -1443,7 +1437,7 @@ PlaylistItemList Playlist::RemoveItemsWithoutUndo(int row, int count) {
 
     if (item->source() == Song::Source_Collection) {
       int id = item->Metadata().id();
-      if (id != -1) {
+      if (id != -1 && collection_items_by_id_.contains(id, item)) {
         collection_items_by_id_.remove(id, item);
       }
     }
@@ -1497,7 +1491,7 @@ void Playlist::SetStreamMetadata(const QUrl &url, const Song &song, const bool m
 
   //qLog(Debug) << "Setting temporary metadata for" << url;
 
-  bool length_changed = song.length_nanosec() != current_item_metadata().length_nanosec();
+  bool update_scrobble_point = song.length_nanosec() != current_item_metadata().length_nanosec();
 
   current_item()->SetTemporaryMetadata(song);
 
@@ -1512,10 +1506,11 @@ void Playlist::SetStreamMetadata(const QUrl &url, const Song &song, const bool m
     }
   }
   else {
+    update_scrobble_point = true;
     InformOfCurrentSongChange();
   }
 
-  if (length_changed) UpdateScrobblePoint();
+  if (update_scrobble_point) UpdateScrobblePoint();
 
 }
 
@@ -1648,6 +1643,11 @@ void Playlist::Shuffle() {
   PlaylistItemList new_items(items_);
 
   int begin = 0;
+  if (current_item_index_.isValid()) {
+    if (new_items[0] != new_items[current_item_index_.row()])
+      std::swap(new_items[0], new_items[current_item_index_.row()]);
+    begin = 1;
+  }
 
   const int count = items_.count();
   for (int i = begin; i < count; ++i) {
@@ -1755,22 +1755,26 @@ void Playlist::set_sequence(PlaylistSequence *v) {
 QSortFilterProxyModel *Playlist::proxy() const { return proxy_; }
 
 SongList Playlist::GetAllSongs() const {
+
   SongList ret;
-  for (const PlaylistItemPtr item : items_) {
+  for (PlaylistItemPtr item : items_) {
     ret << item->Metadata();
   }
   return ret;
+
 }
 
 PlaylistItemList Playlist::GetAllItems() const { return items_; }
 
 quint64 Playlist::GetTotalLength() const {
+
   quint64 ret = 0;
-  for (const PlaylistItemPtr item : items_) {
+  for (PlaylistItemPtr item : items_) {
     quint64 length = item->Metadata().length_nanosec();
     if (length > 0) ret += length;
   }
   return ret;
+
 }
 
 PlaylistItemList Playlist::collection_items_by_id(int id) const {
@@ -1778,30 +1782,38 @@ PlaylistItemList Playlist::collection_items_by_id(int id) const {
 }
 
 void Playlist::TracksAboutToBeDequeued(const QModelIndex&, int begin, int end) {
+
   for (int i = begin; i <= end; ++i) {
     temp_dequeue_change_indexes_ << queue_->mapToSource(queue_->index(i, Column_Title));
   }
+
 }
 
 void Playlist::TracksDequeued() {
+
   for (const QModelIndex &index : temp_dequeue_change_indexes_) {
     emit dataChanged(index, index);
   }
   temp_dequeue_change_indexes_.clear();
   emit QueueChanged();
+
 }
 
 void Playlist::TracksEnqueued(const QModelIndex&, int begin, int end) {
+
   const QModelIndex &b = queue_->mapToSource(queue_->index(begin, Column_Title));
   const QModelIndex &e = queue_->mapToSource(queue_->index(end, Column_Title));
   emit dataChanged(b, e);
+
 }
 
 void Playlist::QueueLayoutChanged() {
+
   for (int i = 0; i < queue_->rowCount(); ++i) {
     const QModelIndex &index = queue_->mapToSource(queue_->index(i, Column_Title));
     emit dataChanged(index, index);
   }
+
 }
 
 void Playlist::ItemChanged(PlaylistItemPtr item) {
@@ -1858,6 +1870,7 @@ void Playlist::InvalidateDeletedSongs() {
 }
 
 void Playlist::RemoveDeletedSongs() {
+
   QList<int> rows_to_remove;
 
   for (int row = 0; row < items_.count(); ++row) {
@@ -1892,7 +1905,7 @@ struct SongSimilarEqual {
 void Playlist::RemoveDuplicateSongs() {
 
   QList<int> rows_to_remove;
-  unordered_map<Song, int, SongSimilarHash, SongSimilarEqual> unique_songs;
+  std::unordered_map<Song, int, SongSimilarHash, SongSimilarEqual> unique_songs;
 
   for (int row = 0; row < items_.count(); ++row) {
     PlaylistItemPtr item = items_[row];
@@ -2003,7 +2016,21 @@ void Playlist::UpdateScrobblePoint(const qint64 seek_point_nanosec) {
     }
   }
 
+  nowplaying_ = false;
   scrobbled_ = false;
 
 }
 
+void Playlist::AlbumCoverLoaded(const Song &song, const AlbumCoverLoaderResult &result) {
+
+  // Update art_manual for local songs that are not in the collection.
+  if (((result.type == AlbumCoverLoaderResult::Type_Manual && result.cover_url.isLocalFile()) || result.type == AlbumCoverLoaderResult::Type_ManuallyUnset) && (song.source() == Song::Source_LocalFile || song.source() == Song::Source_CDDA || song.source() == Song::Source_Device)) {
+    PlaylistItemPtr item = current_item();
+    if (item && item->Metadata() == song && (!item->Metadata().art_manual_is_valid() || (result.type == AlbumCoverLoaderResult::Type_ManuallyUnset && !item->Metadata().has_manually_unset_cover()))) {
+      qLog(Debug) << "Updating art manual for local song" << song.title() << song.album() << song.title() << "to" << result.cover_url << "in playlist.";
+      item->SetArtManual(result.cover_url);
+      Save();
+    }
+  }
+
+}
