@@ -42,7 +42,6 @@
 #include <QString>
 #include <QStringBuilder>
 #include <QUrl>
-#include <QRegExp>
 #include <QIcon>
 #include <QPixmap>
 #include <QPainter>
@@ -328,7 +327,7 @@ QString DateItemDelegate::displayText(const QVariant &value, const QLocale &loca
   if (!ok || time == -1)
     return QString();
 
-  return QDateTime::fromTime_t(time).toString(QLocale::system().dateTimeFormat(QLocale::ShortFormat));
+  return QDateTime::fromSecsSinceEpoch(time).toString(QLocale::system().dateTimeFormat(QLocale::ShortFormat));
 
 }
 
@@ -363,7 +362,7 @@ QWidget *TextItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewI
   return new QLineEdit(parent);
 }
 
-TagCompletionModel::TagCompletionModel(CollectionBackend *backend, Playlist::Column column) : QStringListModel() {
+TagCompletionModel::TagCompletionModel(CollectionBackend *backend, Playlist::Column column) {
 
   QString col = database_column(column);
   if (!col.isEmpty()) {
