@@ -39,10 +39,10 @@ class PlaylistSaveOptionsDialog : public QDialog {
   Q_OBJECT
 
  public:
-  explicit PlaylistSaveOptionsDialog(QWidget *parent = 0);
-  ~PlaylistSaveOptionsDialog();
+  explicit PlaylistSaveOptionsDialog(QWidget *parent = nullptr);
+  ~PlaylistSaveOptionsDialog() override;
 
-  void accept();
+  void accept() override;
   Playlist::Path path_type() const;
 
  private:
