@@ -61,22 +61,22 @@ class Queue : public QAbstractProxyModel {
   void Remove(QList<int> &proxy_rows);
 
   // QAbstractProxyModel
-  void setSourceModel(QAbstractItemModel *source_model);
-  QModelIndex mapFromSource(const QModelIndex &source_index) const;
-  QModelIndex mapToSource(const QModelIndex &proxy_index) const;
+  void setSourceModel(QAbstractItemModel *source_model) override;
+  QModelIndex mapFromSource(const QModelIndex &source_index) const override;
+  QModelIndex mapToSource(const QModelIndex &proxy_index) const override;
 
   // QAbstractItemModel
-  QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
-  QModelIndex parent(const QModelIndex &child) const;
-  int rowCount(const QModelIndex &parent = QModelIndex()) const;
-  int columnCount(const QModelIndex &parent = QModelIndex()) const;
-  QVariant data(const QModelIndex &proxy_index, int role) const;
-  QVariant headerData(int section, Qt::Orientation orientation, int role) const;
-  QStringList mimeTypes() const;
-  Qt::DropActions supportedDropActions() const;
-  QMimeData *mimeData(const QModelIndexList &indexes) const;
-  bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent);
-  Qt::ItemFlags flags(const QModelIndex &index) const;
+  QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
+  QModelIndex parent(const QModelIndex &child) const override;
+  int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+  int columnCount(const QModelIndex &parent = QModelIndex()) const override;
+  QVariant data(const QModelIndex &proxy_index, int role) const override;
+  QVariant headerData(int section, Qt::Orientation orientation, int role) const override;
+  QStringList mimeTypes() const override;
+  Qt::DropActions supportedDropActions() const override;
+  QMimeData *mimeData(const QModelIndexList &indexes) const override;
+  bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent) override;
+  Qt::ItemFlags flags(const QModelIndex &index) const override;
 
  public slots:
   void UpdateSummaryText();
