@@ -46,11 +46,11 @@
 #include <QtDebug>
 
 #include "core/application.h"
-#include "core/closure.h"
 #include "core/network.h"
 #include "core/song.h"
 #include "core/timeconstants.h"
 #include "core/logging.h"
+#include "core/closure.h"
 #include "internet/localredirectserver.h"
 #include "settings/scrobblersettingspage.h"
 
@@ -77,9 +77,25 @@ ScrobblingAPI20::ScrobblingAPI20(const QString &name, const QString &settings_gr
   enabled_(false),
   subscriber_(false),
   submitted_(false),
+  scrobbled_(false),
   timestamp_(0) {}
 
-ScrobblingAPI20::~ScrobblingAPI20() {}
+ScrobblingAPI20::~ScrobblingAPI20() {
+
+  while (!replies_.isEmpty()) {
+    QNetworkReply *reply = replies_.takeFirst();
+    disconnect(reply, nullptr, this, nullptr);
+    reply->abort();
+    reply->deleteLater();
+  }
+
+  if (server_) {
+    disconnect(server_, nullptr, this, nullptr);
+    if (server_->isListening()) server_->close();
+    server_->deleteLater();
+  }
+
+}
 
 void ScrobblingAPI20::ReloadSettings() {
 
@@ -125,7 +141,8 @@ void ScrobblingAPI20::Logout() {
 void ScrobblingAPI20::Authenticate(const bool https) {
 
   if (!server_) {
-    server_ = new LocalRedirectServer(https, this);
+    server_ = new LocalRedirectServer(this);
+    server_->set_https(https);
     if (!server_->Listen()) {
       AuthError(server_->error());
       delete server_;
@@ -228,14 +245,22 @@ void ScrobblingAPI20::RequestSession(const QString &token) {
   session_url.setQuery(session_url_query);
 
   QNetworkRequest req(session_url);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
+  req.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
+#else
   req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
+#endif
   QNetworkReply *reply = network()->get(req);
-  NewClosure(reply, SIGNAL(finished()), this, SLOT(AuthenticateReplyFinished(QNetworkReply*)), reply);
+  replies_ << reply;
+  connect(reply, &QNetworkReply::finished, [=] { AuthenticateReplyFinished(reply); });
 
 }
 
 void ScrobblingAPI20::AuthenticateReplyFinished(QNetworkReply *reply) {
 
+  if (!replies_.contains(reply)) return;
+  replies_.removeAll(reply);
+  disconnect(reply, nullptr, this, nullptr);
   reply->deleteLater();
 
   QByteArray data;
@@ -356,12 +381,17 @@ QNetworkReply *ScrobblingAPI20::CreateRequest(const ParamList &request_params) {
 
   QUrl url(api_url_);
   QNetworkRequest req(url);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
+  req.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
+#else
   req.setAttribute(QNetworkRequest::FollowRedirectsAttribute, true);
+#endif
   req.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
   QByteArray query = url_query.toString(QUrl::FullyEncoded).toUtf8();
   QNetworkReply *reply = network()->post(req, query);
+  replies_ << reply;
 
-  //qLog(Debug) << name_ << "Sending request" << query;
+  //qLog(Debug) << name_ << "Sending request" << url_query.toString(QUrl::FullyDecoded);
 
   return reply;
 
@@ -406,7 +436,10 @@ QByteArray ScrobblingAPI20::GetReplyData(QNetworkReply *reply) {
           reply->error() == QNetworkReply::ContentOperationNotPermittedError ||
           reply->error() == QNetworkReply::AuthenticationRequiredError ||
           error_code == ScrobbleErrorCode::InvalidSessionKey ||
-          error_code == ScrobbleErrorCode::AuthenticationFailed
+          error_code == ScrobbleErrorCode::UnauthorizedToken ||
+          error_code == ScrobbleErrorCode::LoginRequired ||
+          error_code == ScrobbleErrorCode::AuthenticationFailed ||
+          error_code == ScrobbleErrorCode::APIKeySuspended
         ){
         // Session is probably expired
         Logout();
@@ -422,8 +455,11 @@ QByteArray ScrobblingAPI20::GetReplyData(QNetworkReply *reply) {
 
 void ScrobblingAPI20::UpdateNowPlaying(const Song &song) {
 
+  CheckScrobblePrevSong();
+
   song_playing_ = song;
-  timestamp_ = QDateTime::currentDateTime().toTime_t();
+  timestamp_ = QDateTime::currentDateTime().toSecsSinceEpoch();
+  scrobbled_ = false;
 
   if (!IsAuthenticated() || !song.is_metadata_good() || app_->scrobbler()->IsOffline()) return;
 
@@ -443,12 +479,15 @@ void ScrobblingAPI20::UpdateNowPlaying(const Song &song) {
     params << Param("albumArtist", song.albumartist());
 
   QNetworkReply *reply = CreateRequest(params);
-  NewClosure(reply, SIGNAL(finished()), this, SLOT(UpdateNowPlayingRequestFinished(QNetworkReply*)), reply);
+  connect(reply, &QNetworkReply::finished, [=] { UpdateNowPlayingRequestFinished(reply); });
 
 }
 
 void ScrobblingAPI20::UpdateNowPlayingRequestFinished(QNetworkReply *reply) {
 
+  if (!replies_.contains(reply)) return;
+  replies_.removeAll(reply);
+  disconnect(reply, nullptr, this, nullptr);
   reply->deleteLater();
 
   QByteArray data = GetReplyData(reply);
@@ -477,19 +516,27 @@ void ScrobblingAPI20::UpdateNowPlayingRequestFinished(QNetworkReply *reply) {
 }
 
 void ScrobblingAPI20::ClearPlaying() {
+
+  CheckScrobblePrevSong();
+
   song_playing_ = Song();
+  scrobbled_ = false;
+  timestamp_ = 0;
+
 }
 
 void ScrobblingAPI20::Scrobble(const Song &song) {
 
   if (song.id() != song_playing_.id() || song.url() != song_playing_.url() || !song.is_metadata_good()) return;
 
+  scrobbled_ = true;
+
   cache()->Add(song, timestamp_);
 
   if (app_->scrobbler()->IsOffline()) return;
 
   if (!IsAuthenticated()) {
-    emit ErrorMessage(tr("Scrobbler %1 is not authenticated!").arg(name_));
+    if (app_->scrobbler()->ShowErrorDialog()) { emit ErrorMessage(tr("Scrobbler %1 is not authenticated!").arg(name_)); }
     return;
   }
 
@@ -527,16 +574,15 @@ void ScrobblingAPI20::Submit() {
 
   ParamList params = ParamList() << Param("method", "track.scrobble");
 
-  int i(0);
+  int i = 0;
   QList<quint64> list;
-  for (ScrobblerCacheItem *item : cache()->List()) {
+  for (ScrobblerCacheItemPtr item : cache()->List()) {
     if (item->sent_) continue;
     item->sent_ = true;
     if (!batch_) {
       SendSingleScrobble(item);
       continue;
     }
-    i++;
     list << item->timestamp_;
     params << Param(QString("%1[%2]").arg("artist").arg(i), prefer_albumartist_ ? item->effective_albumartist() : item->artist_);
     params << Param(QString("%1[%2]").arg("track").arg(i), item->song_);
@@ -547,19 +593,23 @@ void ScrobblingAPI20::Submit() {
     if (!prefer_albumartist_ && !item->albumartist_.isEmpty() && item->albumartist_.toLower() != Song::kVariousArtists)
       params << Param(QString("%1[%2]").arg("albumArtist").arg(i), item->albumartist_);
     if (item->track_ > 0)
-      params << Param(QString("%1[%2]").arg(i).arg("trackNumber"), QString::number(item->track_));
+      params << Param(QString("%1[%2]").arg("trackNumber").arg(i), QString::number(item->track_));
+    ++i;
     if (i >= kScrobblesPerRequest) break;
   }
 
   if (!batch_ || i <= 0) return;
 
   QNetworkReply *reply = CreateRequest(params);
-  NewClosure(reply, SIGNAL(finished()), this, SLOT(ScrobbleRequestFinished(QNetworkReply*, QList<quint64>)), reply, list);
+  connect(reply, &QNetworkReply::finished, [=] { ScrobbleRequestFinished(reply, list); });
 
 }
 
 void ScrobblingAPI20::ScrobbleRequestFinished(QNetworkReply *reply, QList<quint64> list) {
 
+  if (!replies_.contains(reply)) return;
+  replies_.removeAll(reply);
+  disconnect(reply, nullptr, this, nullptr);
   reply->deleteLater();
 
   QByteArray data = GetReplyData(reply);
@@ -595,15 +645,15 @@ void ScrobblingAPI20::ScrobbleRequestFinished(QNetworkReply *reply, QList<quint6
 
   cache()->Flush(list);
 
-  QJsonValue json_scrobbles = json_obj["scrobbles"];
-  if (!json_scrobbles.isObject()) {
+  QJsonValue value_scrobbles = json_obj["scrobbles"];
+  if (!value_scrobbles.isObject()) {
     Error("Json scrobbles is not an object.", json_obj);
     DoSubmit();
     return;
   }
-  json_obj = json_scrobbles.toObject();
+  json_obj = value_scrobbles.toObject();
   if (json_obj.isEmpty()) {
-    Error("Json scrobbles object is empty.", json_scrobbles);
+    Error("Json scrobbles object is empty.", value_scrobbles);
     DoSubmit();
     return;
   }
@@ -613,41 +663,55 @@ void ScrobblingAPI20::ScrobbleRequestFinished(QNetworkReply *reply, QList<quint6
     return;
   }
 
-  QJsonValue json_attr = json_obj["@attr"];
-  if (!json_attr.isObject()) {
-    Error("Json scrobbles attr is not an object.", json_attr);
+  QJsonValue value_attr = json_obj["@attr"];
+  if (!value_attr.isObject()) {
+    Error("Json scrobbles attr is not an object.", value_attr);
     DoSubmit();
     return;
   }
-  QJsonObject json_obj_attr = json_attr.toObject();
-  if (json_obj_attr.isEmpty()) {
-    Error("Json scrobbles attr is empty.", json_attr);
+  QJsonObject obj_attr = value_attr.toObject();
+  if (obj_attr.isEmpty()) {
+    Error("Json scrobbles attr is empty.", value_attr);
     DoSubmit();
     return;
   }
-  if (!json_obj_attr.contains("accepted") || !json_obj_attr.contains("ignored")) {
-    Error("Json scrobbles attr is missing values.", json_obj_attr);
+  if (!obj_attr.contains("accepted") || !obj_attr.contains("ignored")) {
+    Error("Json scrobbles attr is missing values.", obj_attr);
     DoSubmit();
     return;
   }
-  int accepted = json_obj_attr["accepted"].toInt();
-  int ignored = json_obj_attr["ignored"].toInt();
+  int accepted = obj_attr["accepted"].toInt();
+  int ignored = obj_attr["ignored"].toInt();
+
   qLog(Debug) << name_ << "Scrobbles accepted:" << accepted << "ignored:" << ignored;
 
-  QJsonValue json_scrobble = json_obj["scrobble"];
-  if (!json_scrobble.isArray()) {
-    Error("Json scrobbles scrobble is not array.", json_scrobble);
+  QJsonArray array_scrobble;
+
+  QJsonValue value_scrobble = json_obj["scrobble"];
+  if (value_scrobble.isObject()) {
+    QJsonObject obj_scrobble = value_scrobble.toObject();
+    if (obj_scrobble.isEmpty()) {
+      Error("Json scrobbles scrobble object is empty.", obj_scrobble);
+      DoSubmit();
+      return;
+    }
+    array_scrobble.append(obj_scrobble);
+  }
+  else if (value_scrobble.isArray()) {
+    array_scrobble = value_scrobble.toArray();
+    if (array_scrobble.isEmpty()) {
+      Error("Json scrobbles scrobble array is empty.", value_scrobble);
+      DoSubmit();
+      return;
+    }
+  }
+  else {
+    Error("Json scrobbles scrobble is not an object or array.", value_scrobble);
     DoSubmit();
     return;
   }
-  QJsonArray json_array_scrobble = json_scrobble.toArray();
-  if (json_array_scrobble.isEmpty()) {
-    Error("Json scrobbles scrobble array is empty.", json_scrobble);
-    DoSubmit();
-    return;
-  }
-  
-  for (const QJsonValue &value : json_array_scrobble) {
+
+  for (const QJsonValue &value : array_scrobble) {
 
     if (!value.isObject()) {
       Error("Json scrobbles scrobble array value is not an object.", value);
@@ -669,37 +733,36 @@ void ScrobblingAPI20::ScrobbleRequestFinished(QNetworkReply *reply, QList<quint6
       continue;
     }
 
-    QJsonValue json_value_artist = json_track["artist"];
-    QJsonValue json_value_album = json_track["album"];
-    QJsonValue json_value_song = json_track["track"];
-    QJsonValue json_value_ignoredmessage = json_track["ignoredMessage"];
+    QJsonValue value_artist = json_track["artist"];
+    QJsonValue value_album = json_track["album"];
+    QJsonValue value_song = json_track["track"];
+    QJsonValue value_ignoredmessage = json_track["ignoredMessage"];
     //quint64 timestamp = json_track["timestamp"].toVariant().toULongLong();
 
-    if (!json_value_artist.isObject() || !json_value_album.isObject() || !json_value_song.isObject() || !json_value_ignoredmessage.isObject()) {
+    if (!value_artist.isObject() || !value_album.isObject() || !value_song.isObject() || !value_ignoredmessage.isObject()) {
       Error("Json scrobbles scrobble values are not objects.", json_track);
       continue;
     }
 
-    QJsonObject json_obj_artist = json_value_artist.toObject();
-    QJsonObject json_obj_album = json_value_album.toObject();
-    QJsonObject json_obj_song = json_value_song.toObject();
-    QJsonObject json_obj_ignoredmessage = json_value_ignoredmessage.toObject();
+    QJsonObject obj_artist = value_artist.toObject();
+    QJsonObject obj_album = value_album.toObject();
+    QJsonObject obj_song = value_song.toObject();
+    QJsonObject obj_ignoredmessage = value_ignoredmessage.toObject();
 
-    if (json_obj_artist.isEmpty() || json_obj_album.isEmpty() || json_obj_song.isEmpty() || json_obj_ignoredmessage.isEmpty()) {
+    if (obj_artist.isEmpty() || obj_album.isEmpty() || obj_song.isEmpty() || obj_ignoredmessage.isEmpty()) {
       Error("Json scrobbles scrobble values objects are empty.", json_track);
       continue;
     }
 
-    if (!json_obj_artist.contains("#text") || !json_obj_album.contains("#text") || !json_obj_song.contains("#text")) {
-      // Just ignore this, as Last.fm seem to return 1 ignored scrobble for a blank song for each request (no idea why).
+    if (!obj_artist.contains("#text") || !obj_album.contains("#text") || !obj_song.contains("#text")) {
       continue;
     }
 
-    QString artist = json_obj_artist["#text"].toString();
-    QString album = json_obj_album["#text"].toString();
-    QString song = json_obj_song["#text"].toString();
-    bool ignoredmessage = json_obj_ignoredmessage["code"].toVariant().toBool();
-    QString ignoredmessage_text = json_obj_ignoredmessage["#text"].toString();
+    QString artist = obj_artist["#text"].toString();
+    QString album = obj_album["#text"].toString();
+    QString song = obj_song["#text"].toString();
+    bool ignoredmessage = obj_ignoredmessage["code"].toVariant().toBool();
+    QString ignoredmessage_text = obj_ignoredmessage["#text"].toString();
 
     if (ignoredmessage) {
       Error(QString("Scrobble for \"%1\" ignored: %2").arg(song).arg(ignoredmessage_text));
@@ -714,7 +777,7 @@ void ScrobblingAPI20::ScrobbleRequestFinished(QNetworkReply *reply, QList<quint6
 
 }
 
-void ScrobblingAPI20::SendSingleScrobble(ScrobblerCacheItem *item) {
+void ScrobblingAPI20::SendSingleScrobble(ScrobblerCacheItemPtr item) {
 
   ParamList params = ParamList()
     << Param("method", "track.scrobble")
@@ -731,15 +794,18 @@ void ScrobblingAPI20::SendSingleScrobble(ScrobblerCacheItem *item) {
     params << Param("trackNumber", QString::number(item->track_));
 
   QNetworkReply *reply = CreateRequest(params);
-  NewClosure(reply, SIGNAL(finished()), this, SLOT(SingleScrobbleRequestFinished(QNetworkReply*, quint64)), reply, item->timestamp_);
+  connect(reply, &QNetworkReply::finished, [=] { SingleScrobbleRequestFinished(reply, item->timestamp_); });
 
 }
 
 void ScrobblingAPI20::SingleScrobbleRequestFinished(QNetworkReply *reply, quint64 timestamp) {
 
+  if (!replies_.contains(reply)) return;
+  replies_.removeAll(reply);
+  disconnect(reply, nullptr, this, nullptr);
   reply->deleteLater();
 
-  ScrobblerCacheItem *item = cache()->Get(timestamp);
+  ScrobblerCacheItemPtr item = cache()->Get(timestamp);
   if (!item) {
     Error(QString("Received reply for non-existing cache entry %1.").arg(timestamp));
     return;
@@ -775,14 +841,14 @@ void ScrobblingAPI20::SingleScrobbleRequestFinished(QNetworkReply *reply, quint6
   cache()->Remove(timestamp);
   item = nullptr;
 
-  QJsonValue json_scrobbles = json_obj["scrobbles"];
-  if (!json_scrobbles.isObject()) {
+  QJsonValue value_scrobbles = json_obj["scrobbles"];
+  if (!value_scrobbles.isObject()) {
     Error("Json scrobbles is not an object.", json_obj);
     return;
   }
-  json_obj = json_scrobbles.toObject();
+  json_obj = value_scrobbles.toObject();
   if (json_obj.isEmpty()) {
-    Error("Json scrobbles object is empty.", json_scrobbles);
+    Error("Json scrobbles object is empty.", value_scrobbles);
     return;
   }
   if (!json_obj.contains("@attr") || !json_obj.contains("scrobble")) {
@@ -790,30 +856,30 @@ void ScrobblingAPI20::SingleScrobbleRequestFinished(QNetworkReply *reply, quint6
     return;
   }
 
-  QJsonValue json_attr = json_obj["@attr"];
-  if (!json_attr.isObject()) {
-    Error("Json scrobbles attr is not an object.", json_attr);
+  QJsonValue value_attr = json_obj["@attr"];
+  if (!value_attr.isObject()) {
+    Error("Json scrobbles attr is not an object.", value_attr);
     return;
   }
-  QJsonObject json_obj_attr = json_attr.toObject();
-  if (json_obj_attr.isEmpty()) {
-    Error("Json scrobbles attr is empty.", json_attr);
+  QJsonObject obj_attr = value_attr.toObject();
+  if (obj_attr.isEmpty()) {
+    Error("Json scrobbles attr is empty.", value_attr);
     return;
   }
 
-  QJsonValue json_scrobble = json_obj["scrobble"];
-  if (!json_scrobble.isObject()) {
-    Error("Json scrobbles scrobble is not an object.", json_scrobble);
+  QJsonValue value_scrobble = json_obj["scrobble"];
+  if (!value_scrobble.isObject()) {
+    Error("Json scrobbles scrobble is not an object.", value_scrobble);
     return;
   }
-  QJsonObject json_obj_scrobble = json_scrobble.toObject();
+  QJsonObject json_obj_scrobble = value_scrobble.toObject();
   if (json_obj_scrobble.isEmpty()) {
-    Error("Json scrobbles scrobble is empty.", json_scrobble);
+    Error("Json scrobbles scrobble is empty.", value_scrobble);
     return;
   }
 
-  if (!json_obj_attr.contains("accepted") || !json_obj_attr.contains("ignored")) {
-    Error("Json scrobbles attr is missing values.", json_obj_attr);
+  if (!obj_attr.contains("accepted") || !obj_attr.contains("ignored")) {
+    Error("Json scrobbles attr is missing values.", obj_attr);
     return;
   }
 
@@ -849,7 +915,7 @@ void ScrobblingAPI20::SingleScrobbleRequestFinished(QNetworkReply *reply, quint6
   QString album = json_obj_album["#text"].toString();
   QString song = json_obj_song["#text"].toString();
 
-  int accepted = json_obj_attr["accepted"].toVariant().toInt();
+  int accepted = obj_attr["accepted"].toVariant().toInt();
   if (accepted == 1) {
     qLog(Debug) << name_ << "Scrobble for" << song << "accepted";
   }
@@ -879,12 +945,15 @@ void ScrobblingAPI20::Love() {
     params << Param("albumArtist", song_playing_.albumartist());
 
   QNetworkReply *reply = CreateRequest(params);
-  NewClosure(reply, SIGNAL(finished()), this, SLOT(LoveRequestFinished(QNetworkReply*)), reply);
+  connect(reply, &QNetworkReply::finished, [=] { LoveRequestFinished(reply); });
 
 }
 
 void ScrobblingAPI20::LoveRequestFinished(QNetworkReply *reply) {
 
+  if (!replies_.contains(reply)) return;
+  replies_.removeAll(reply);
+  disconnect(reply, nullptr, this, nullptr);
   reply->deleteLater();
 
   QByteArray data = GetReplyData(reply);
@@ -929,8 +998,6 @@ void ScrobblingAPI20::LoveRequestFinished(QNetworkReply *reply) {
     }
   }
 
-  qLog(Debug) << name_ << json_obj;
-
 }
 
 void ScrobblingAPI20::AuthError(const QString &error) {
@@ -942,41 +1009,83 @@ void ScrobblingAPI20::Error(const QString &error, const QVariant &debug) {
   qLog(Error) << name_ << error;
   if (debug.isValid()) qLog(Debug) << debug;
 
+  if (app_->scrobbler()->ShowErrorDialog()) { emit ErrorMessage(tr("Scrobbler %1 error: %2").arg(name_).arg(error)); }
+
 }
 
 QString ScrobblingAPI20::ErrorString(const ScrobbleErrorCode error) const {
 
   switch (error) {
+    case ScrobbleErrorCode::NoError:
+      return QString("This error does not exist.");
     case ScrobbleErrorCode::InvalidService:
-      return QString("Invalid service - This service does not exist");
+      return QString("Invalid service - This service does not exist.");
     case ScrobbleErrorCode::InvalidMethod:
-      return QString("Invalid Method - No method with that name in this package");
+      return QString("Invalid Method - No method with that name in this package.");
     case ScrobbleErrorCode::AuthenticationFailed:
-      return QString("Authentication Failed - You do not have permissions to access the service");
+      return QString("Authentication Failed - You do not have permissions to access the service.");
     case ScrobbleErrorCode::InvalidFormat:
-      return QString("Invalid format - This service doesn't exist in that format");
+      return QString("Invalid format - This service doesn't exist in that format.");
     case ScrobbleErrorCode::InvalidParameters:
-      return QString("Invalid parameters - Your request is missing a required parameter");
+      return QString("Invalid parameters - Your request is missing a required parameter.");
     case ScrobbleErrorCode::InvalidResourceSpecified:
       return QString("Invalid resource specified");
     case ScrobbleErrorCode::OperationFailed:
-      return QString("Operation failed - Something else went wrong");
+      return QString("Operation failed - Most likely the backend service failed. Please try again.");
     case ScrobbleErrorCode::InvalidSessionKey:
-      return QString("Invalid session key - Please re-authenticate");
+      return QString("Invalid session key - Please re-authenticate.");
     case ScrobbleErrorCode::InvalidApiKey:
-      return QString("Invalid API key - You must be granted a valid key by last.fm");
+      return QString("Invalid API key - You must be granted a valid key by last.fm.");
     case ScrobbleErrorCode::ServiceOffline:
       return QString("Service Offline - This service is temporarily offline. Try again later.");
+    case ScrobbleErrorCode::SubscribersOnly:
+      return QString("Subscribers Only - This station is only available to paid last.fm subscribers.");
     case ScrobbleErrorCode::InvalidMethodSignature:
-        return QString("Invalid method signature supplied");
-    case ScrobbleErrorCode::TempError:
-      return QString("There was a temporary error processing your request. Please try again");
-    case ScrobbleErrorCode::SuspendedAPIKey:
+      return QString("Invalid method signature supplied.");
+    case ScrobbleErrorCode::UnauthorizedToken:
+      return QString("Unauthorized Token - This token has not been authorized.");
+    case ScrobbleErrorCode::ItemUnavailable:
+      return QString("This item is not available for streaming.");
+    case ScrobbleErrorCode::TemporarilyUnavailable:
+      return QString("The service is temporarily unavailable, please try again.");
+    case ScrobbleErrorCode::LoginRequired:
+      return QString("Login: User requires to be logged in.");
+    case ScrobbleErrorCode::TrialExpired:
+      return QString("Trial Expired - This user has no free radio plays left. Subscription required.");
+    case ScrobbleErrorCode::ErrorDoesNotExist:
+      return QString("This error does not exist.");
+    case ScrobbleErrorCode::NotEnoughContent:
+      return QString("Not Enough Content - There is not enough content to play this station.");
+    case ScrobbleErrorCode::NotEnoughMembers:
+      return QString("Not Enough Members - This group does not have enough members for radio.");
+    case ScrobbleErrorCode::NotEnoughFans:
+      return QString("Not Enough Fans - This artist does not have enough fans for for radio.");
+    case ScrobbleErrorCode::NotEnoughNeighbours:
+      return QString("Not Enough Neighbours - There are not enough neighbours for radio.");
+    case ScrobbleErrorCode::NoPeakRadio:
+      return QString("No Peak Radio - This user is not allowed to listen to radio during peak usage.");
+    case ScrobbleErrorCode::RadioNotFound:
+      return QString("Radio Not Found - Radio station not found.");
+    case ScrobbleErrorCode::APIKeySuspended:
       return QString("Suspended API key - Access for your account has been suspended, please contact Last.fm");
+    case ScrobbleErrorCode::Deprecated:
+      return QString("Deprecated - This type of request is no longer supported.");
     case ScrobbleErrorCode::RateLimitExceeded:
-      return QString("Rate limit exceeded - Your IP has made too many requests in a short period");
-    default:
-      return QString("Unknown error");
+      return QString("Rate limit exceeded - Your IP has made too many requests in a short period.");
+  }
+
+  return QString("Unknown error.");
+
+}
+
+void ScrobblingAPI20::CheckScrobblePrevSong() {
+
+  quint64 duration = QDateTime::currentDateTime().toSecsSinceEpoch() - timestamp_;
+
+  if (!scrobbled_ && song_playing_.is_metadata_good() && song_playing_.source() == Song::Source_Stream && duration > 30) {
+    Song song(song_playing_);
+    song.set_length_nanosec(duration * kNsecPerSec);
+    Scrobble(song);
   }
 
 }
