@@ -26,7 +26,8 @@
 #include <QVariant>
 #include <QString>
 #include <QStringList>
-#include <QRegExp>
+#include <QRegularExpression>
+#include <QRegularExpressionMatch>
 #include <QDir>
 #include <QLocale>
 #include <QSettings>
@@ -36,6 +37,7 @@
 #include <QSpinBox>
 #include <QComboBox>
 #include <QGroupBox>
+#include <QStandardPaths>
 
 #include "core/iconloader.h"
 #include "core/mainwindow.h"
@@ -64,30 +66,29 @@ BehaviourSettingsPage::BehaviourSettingsPage(SettingsDialog *dialog) : SettingsP
   connect(ui_->checkbox_showtrayicon, SIGNAL(toggled(bool)), SLOT(ShowTrayIconToggled(bool)));
 
 #ifdef Q_OS_MACOS
-  ui_->checkbox_showtrayicon->setEnabled(false);
-  ui_->groupbox_startup->setEnabled(false);
-#else
-  if (QSystemTrayIcon::isSystemTrayAvailable()) {
-    ui_->checkbox_showtrayicon->setEnabled(true);
-    ui_->groupbox_startup->setEnabled(true);
-  }
-  else {
-    ui_->checkbox_showtrayicon->setEnabled(false);
-    ui_->groupbox_startup->setEnabled(false);
-  }
+  ui_->checkbox_showtrayicon->hide();
+  ui_->checkbox_keeprunning->hide();
+  ui_->checkbox_scrolltrayicon->hide();
+  ui_->groupbox_startup->hide();
 #endif
 
 #ifdef HAVE_TRANSLATIONS
   // Populate the language combo box.  We do this by looking at all the compiled in translations.
-  QDir dir(":/translations/");
-  QStringList codes(dir.entryList(QStringList() << "*.qm"));
-  QRegExp lang_re("^strawberry_(.*).qm$");
+  QDir dir1(":/translations/");
+  QDir dir2(TRANSLATIONS_DIR);
+  QStringList codes(dir1.entryList(QStringList() << "*.qm"));
+  if (dir2.exists()) {
+    codes << dir2.entryList(QStringList() << "*.qm");
+  }
+  QRegularExpression lang_re("^strawberry_(.*).qm$");
   for (const QString &filename : codes) {
 
-    // The regex captures the "ru" from "strawberry_ru.qm"
-    if (!lang_re.exactMatch(filename)) continue;
+    QRegularExpressionMatch re_match = lang_re.match(filename);
 
-    QString code = lang_re.cap(1);
+    // The regex captures the "ru" from "strawberry_ru.qm"
+    if (!re_match.hasMatch()) continue;
+
+    QString code = re_match.captured(1);
     QString lookup_code = QString(code)
                               .replace("@latin", "_Latn")
                               .replace("_CN", "_Hans_CN")
@@ -150,31 +151,56 @@ void BehaviourSettingsPage::Load() {
   QSettings s;
 
   s.beginGroup(kSettingsGroup);
-#ifdef Q_OS_MACOS
-  ui_->checkbox_showtrayicon->setChecked(false);
-  ui_->checkbox_scrolltrayicon->setChecked(false);
-  ui_->checkbox_keeprunning->setChecked(false);
-#else
+#ifndef Q_OS_MACOS
   if (QSystemTrayIcon::isSystemTrayAvailable()) {
+    ui_->checkbox_showtrayicon->setEnabled(true);
+    ui_->checkbox_keeprunning->setEnabled(true);
+    ui_->checkbox_scrolltrayicon->setEnabled(true);
+    ui_->radiobutton_hide->setEnabled(true);
     ui_->checkbox_showtrayicon->setChecked(s.value("showtrayicon", true).toBool());
-    ui_->checkbox_scrolltrayicon->setChecked(s.value("scrolltrayicon", ui_->checkbox_showtrayicon->isChecked()).toBool());
     ui_->checkbox_keeprunning->setChecked(s.value("keeprunning", false).toBool());
+    ui_->checkbox_scrolltrayicon->setChecked(s.value("scrolltrayicon", ui_->checkbox_showtrayicon->isChecked()).toBool());
   }
   else {
+    ui_->checkbox_showtrayicon->setEnabled(false);
+    ui_->checkbox_keeprunning->setEnabled(false);
+    ui_->checkbox_scrolltrayicon->setEnabled(false);
+    ui_->radiobutton_hide->setEnabled(false);
     ui_->checkbox_showtrayicon->setChecked(false);
-    ui_->checkbox_scrolltrayicon->setChecked(false);
     ui_->checkbox_keeprunning->setChecked(false);
+    ui_->checkbox_scrolltrayicon->setChecked(false);
+    ui_->radiobutton_hide->setChecked(false);
   }
 #endif
+
   ui_->checkbox_resumeplayback->setChecked(s.value("resumeplayback", false).toBool());
   ui_->checkbox_playingwidget->setChecked(s.value("playing_widget", true).toBool());
 
-  MainWindow::StartupBehaviour behaviour = MainWindow::StartupBehaviour(s.value("startupbehaviour", MainWindow::Startup_Remember).toInt());
+#ifndef Q_OS_MACOS
+  StartupBehaviour behaviour = StartupBehaviour(s.value("startupbehaviour", Startup_Remember).toInt());
   switch (behaviour) {
-    case MainWindow::Startup_AlwaysHide: ui_->radiobutton_alwayshide->setChecked(true); break;
-    case MainWindow::Startup_AlwaysShow: ui_->radiobutton_alwaysshow->setChecked(true); break;
-    case MainWindow::Startup_Remember:   ui_->radiobutton_remember->setChecked(true); break;
+    case Startup_Show:
+      ui_->radiobutton_show->setChecked(true);
+      break;
+    case Startup_ShowMaximized:
+      ui_->radiobutton_show_maximized->setChecked(true);
+      break;
+    case Startup_ShowMinimized:
+      ui_->radiobutton_show_minimized->setChecked(true);
+      break;
+    case Startup_Hide:
+      if (QSystemTrayIcon::isSystemTrayAvailable()) {
+        ui_->radiobutton_hide->setChecked(true);
+        break;
+      }
+      ;
+      // fallthrough
+    case BehaviourSettingsPage::Startup_Remember:
+    default:
+      ui_->radiobutton_remember->setChecked(true);
+      break;
   }
+#endif
 
   QString name = language_map_.key(s.value("language").toString());
   if (name.isEmpty())
@@ -196,6 +222,8 @@ void BehaviourSettingsPage::Load() {
 
   s.endGroup();
 
+  Init(ui_->layout_behavioursettingspage->parentWidget());
+
 }
 
 void BehaviourSettingsPage::Save() {
@@ -209,10 +237,12 @@ void BehaviourSettingsPage::Save() {
   s.setValue("playing_widget", ui_->checkbox_playingwidget->isChecked());
   s.setValue("scrolltrayicon", ui_->checkbox_scrolltrayicon->isChecked());
 
-  MainWindow::StartupBehaviour behaviour = MainWindow::Startup_Remember;
-  if (ui_->radiobutton_alwayshide->isChecked()) behaviour = MainWindow::Startup_AlwaysHide;
-  if (ui_->radiobutton_alwaysshow->isChecked()) behaviour = MainWindow::Startup_AlwaysShow;
-  if (ui_->radiobutton_remember->isChecked()) behaviour = MainWindow::Startup_Remember;
+  StartupBehaviour behaviour = Startup_Remember;
+  if (ui_->radiobutton_remember->isChecked()) behaviour = Startup_Remember;
+  if (ui_->radiobutton_show->isChecked()) behaviour = Startup_Show;
+  if (ui_->radiobutton_hide->isChecked()) behaviour = Startup_Hide;
+  if (ui_->radiobutton_show_maximized->isChecked()) behaviour = Startup_ShowMaximized;
+  if (ui_->radiobutton_show_minimized->isChecked()) behaviour = Startup_ShowMinimized;
   s.setValue("startupbehaviour", int(behaviour));
 
   s.setValue("language", language_map_.contains(ui_->combobox_language->currentText()) ? language_map_[ui_->combobox_language->currentText()] : QString());
@@ -240,8 +270,8 @@ void BehaviourSettingsPage::Save() {
 
 void BehaviourSettingsPage::ShowTrayIconToggled(bool on) {
 
-  ui_->radiobutton_alwayshide->setEnabled(on);
-  if (!on && ui_->radiobutton_alwayshide->isChecked()) ui_->radiobutton_remember->setChecked(true);
+  ui_->radiobutton_hide->setEnabled(on);
+  if (!on && ui_->radiobutton_hide->isChecked()) ui_->radiobutton_remember->setChecked(true);
   ui_->checkbox_keeprunning->setEnabled(on);
   ui_->checkbox_scrolltrayicon->setEnabled(on);
 
