@@ -23,12 +23,19 @@
 #include <QObject>
 #include <QMessageBox>
 #include <QSettings>
+#include <QCheckBox>
+#include <QLabel>
+#include <QLineEdit>
+#include <QPushButton>
+#include <QSpinBox>
 
+#include "settingsdialog.h"
+#include "settingspage.h"
 #include "core/application.h"
 #include "core/iconloader.h"
+#include "widgets/loginstatewidget.h"
 
 #include "scrobbler/audioscrobbler.h"
-#include "scrobbler/scrobblerservice.h"
 #include "scrobbler/lastfmscrobbler.h"
 #include "scrobbler/librefmscrobbler.h"
 #include "scrobbler/listenbrainzscrobbler.h"
@@ -71,6 +78,8 @@ ScrobblerSettingsPage::ScrobblerSettingsPage(SettingsDialog *parent)
   connect(ui_->widget_listenbrainz_login_state, SIGNAL(LogoutClicked()), SLOT(ListenBrainz_Logout()));
   ui_->widget_listenbrainz_login_state->AddCredentialGroup(ui_->widget_listenbrainz_login);
 
+  ui_->label_listenbrainz_token->setText("<html><head/><body><p>" + tr("Enter your user token from") + " " + "<a href=\"https://listenbrainz.org/profile/\"><span style=\"text-decoration: underline; color:#0000ff;\">https://listenbrainz.org/profile/</span></a></p></body></html>");
+
   resize(sizeHint());
 
 }
@@ -84,6 +93,8 @@ void ScrobblerSettingsPage::Load() {
   ui_->checkbox_love_button->setChecked(scrobbler_->LoveButton());
   ui_->checkbox_offline->setChecked(scrobbler_->IsOffline());
   ui_->spinbox_submit->setValue(scrobbler_->SubmitDelay());
+  ui_->checkbox_albumartist->setChecked(scrobbler_->PreferAlbumArtist());
+  ui_->checkbox_show_error_dialog->setChecked(scrobbler_->ShowErrorDialog());
 
   ui_->checkbox_lastfm_enable->setChecked(lastfmscrobbler_->IsEnabled());
   ui_->checkbox_lastfm_https->setChecked(lastfmscrobbler_->IsUseHTTPS());
@@ -96,6 +107,8 @@ void ScrobblerSettingsPage::Load() {
   ui_->lineedit_listenbrainz_user_token->setText(listenbrainzscrobbler_->user_token());
   ListenBrainz_RefreshControls(listenbrainzscrobbler_->IsAuthenticated());
 
+  Init(ui_->layout_scrobblersettingspage->parentWidget());
+
 }
 
 void ScrobblerSettingsPage::Save() {
@@ -108,6 +121,8 @@ void ScrobblerSettingsPage::Save() {
   s.setValue("love_button", ui_->checkbox_love_button->isChecked());
   s.setValue("offline", ui_->checkbox_offline->isChecked());
   s.setValue("submit", ui_->spinbox_submit->value());
+  s.setValue("albumartist", ui_->checkbox_albumartist->isChecked());
+  s.setValue("show_error_dialog", ui_->checkbox_show_error_dialog->isChecked());
   s.endGroup();
 
   s.beginGroup(LastFMScrobbler::kSettingsGroup);
