@@ -34,7 +34,7 @@
 #include <QSettings>
 
 #include "engine/engine_fwd.h"
-#include "widgets/osd.h"
+#include "osd/osdbase.h"
 
 class QMainWindow;
 class QWidget;
@@ -60,8 +60,8 @@ class Ui_SettingsDialog;
 class SettingsItemDelegate : public QStyledItemDelegate {
  public:
   explicit SettingsItemDelegate(QObject *parent);
-  QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;
-  void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
+  QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override;
+  void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
 };
 
 
@@ -69,8 +69,8 @@ class SettingsDialog : public QDialog {
   Q_OBJECT
 
  public:
-  explicit SettingsDialog(Application *app, QMainWindow *mainwindow, QWidget *parent = nullptr);
-  ~SettingsDialog();
+  explicit SettingsDialog(Application *app, OSDBase *osd, QMainWindow *mainwindow, QWidget *parent = nullptr);
+  ~SettingsDialog() override;
 
   enum Page {
     Page_Behaviour,
@@ -78,15 +78,18 @@ class SettingsDialog : public QDialog {
     Page_Backend,
     Page_Playback,
     Page_Playlist,
-    Page_GlobalShortcuts,
+    Page_Scrobbler,
+    Page_Covers,
+    Page_Lyrics,
+    Page_Transcoding,
+    Page_Proxy,
     Page_Appearance,
     Page_Context,
     Page_Notifications,
-    Page_Transcoding,
-    Page_Proxy,
-    Page_Scrobbler,
+    Page_GlobalShortcuts,
     Page_Moodbar,
-    Page_Subsonic
+    Page_Subsonic,
+    Page_Tidal,
   };
 
   enum Role {
@@ -98,6 +101,7 @@ class SettingsDialog : public QDialog {
   bool is_loading_settings() const { return loading_settings_; }
 
   Application *app() const { return app_; }
+  OSDBase *osd() const { return osd_; }
   Player *player() const { return player_; }
   EngineBase *engine() const { return engine_; }
   CollectionDirectoryModel *collection_directory_model() const { return model_; }
@@ -110,8 +114,8 @@ class SettingsDialog : public QDialog {
   void ComboBoxLoadFromSettings(const QSettings &s, QComboBox *combobox, const QString &setting, const int default_value);
 
  protected:
-  void showEvent(QShowEvent *e);
-  void closeEvent(QCloseEvent*);
+  void showEvent(QShowEvent *e) override;
+  void closeEvent(QCloseEvent*) override;
 
  private:
   struct PageData {
@@ -121,8 +125,8 @@ class SettingsDialog : public QDialog {
   };
 
   // QDialog
-  void accept();
-  void reject();
+  void accept() override;
+  void reject() override;
 
   void LoadGeometry();
   void SaveGeometry();
@@ -130,11 +134,12 @@ class SettingsDialog : public QDialog {
   QTreeWidgetItem *AddCategory(const QString &name);
   void AddPage(Page id, SettingsPage *page, QTreeWidgetItem *parent = nullptr);
 
+  void Apply();
   void Save();
 
  signals:
   void ReloadSettings();
-  void NotificationPreview(OSD::Behaviour, QString, QString);
+  void NotificationPreview(OSDBase::Behaviour, QString, QString);
 
  private slots:
   void CurrentItemChanged(QTreeWidgetItem *item);
@@ -145,6 +150,7 @@ class SettingsDialog : public QDialog {
 
   QMainWindow *mainwindow_;
   Application *app_;
+  OSDBase *osd_;
   Player *player_;
   EngineBase *engine_;
   CollectionDirectoryModel *model_;
